@@ -27,9 +27,18 @@
  *
  **********************************************************************
  *
- * $Id: SQLAllocHandle.c,v 1.10 2006/06/28 08:08:41 lurcher Exp $
+ * $Id: SQLAllocHandle.c,v 1.13 2009/02/18 17:59:08 lurcher Exp $
  *
  * $Log: SQLAllocHandle.c,v $
+ * Revision 1.13  2009/02/18 17:59:08  lurcher
+ * Shift to using config.h, the compile lines were making it hard to spot warnings
+ *
+ * Revision 1.12  2007/12/17 13:13:03  lurcher
+ * Fix a couple of descriptor typo's
+ *
+ * Revision 1.11  2007/02/12 11:49:34  lurcher
+ * Add QT4 support to existing GUI parts
+ *
  * Revision 1.10  2006/06/28 08:08:41  lurcher
  * Add timestamp with timezone to Postgres7.1 driver
  *
@@ -236,13 +245,14 @@
  *
  **********************************************************************/
 
+#include <config.h>
 #include "drivermanager.h"
 #if defined ( COLLECT_STATS ) && defined( HAVE_SYS_SEM_H )
 #include "__stats.h"
 #include <uodbc_stats.h>
 #endif
 
-static char const rcsid[]= "$RCSfile: SQLAllocHandle.c,v $ $Revision: 1.10 $";
+static char const rcsid[]= "$RCSfile: SQLAllocHandle.c,v $ $Revision: 1.13 $";
 
 /*
  * connection pooling stuff
@@ -267,8 +277,15 @@ SQLRETURN __SQLAllocHandle( SQLSMALLINT handle_type,
             DMHENV environment;
             char pooling_string[ 128 ];
 
-            if ( !output_handle )
+            if ( !output_handle ) 
+            {
                 return SQL_ERROR;
+            }
+
+            if ( input_handle )
+            {
+                return SQL_INVALID_HANDLE;
+            }
 
             /*
              * check connection pooling attributes
@@ -732,6 +749,7 @@ SQLRETURN __SQLAllocHandle( SQLSMALLINT handle_type,
                         }
                         statement -> implicit_ard = statement -> ard;
                         statement -> ard -> implicit = 1;
+                        statement -> ard -> associated_with = statement;
                         statement -> ard -> state = STATE_D1i;
                         statement -> ard -> driver_desc = desc;
                         statement -> ard -> connection = connection;
@@ -755,7 +773,7 @@ SQLRETURN __SQLAllocHandle( SQLSMALLINT handle_type,
                          * to wrap around this
                          */
                         statement -> apd = __alloc_desc();
-                        if ( !statement -> ard )
+                        if ( !statement -> apd )
                         {
                             dm_log_write( __FILE__, 
                                 __LINE__, 
@@ -775,6 +793,7 @@ SQLRETURN __SQLAllocHandle( SQLSMALLINT handle_type,
                         }
                         statement -> implicit_apd = statement -> apd;
                         statement -> apd -> implicit = 1;
+                        statement -> apd -> associated_with = statement;
                         statement -> apd -> state = STATE_D1i;
                         statement -> apd -> driver_desc = desc;
                         statement -> apd -> connection = connection;
@@ -818,6 +837,7 @@ SQLRETURN __SQLAllocHandle( SQLSMALLINT handle_type,
                         }
                         statement -> implicit_ird = statement -> ird;
                         statement -> ird -> implicit = 1;
+                        statement -> ird -> associated_with = statement;
                         statement -> ird -> state = STATE_D1i;
                         statement -> ird -> driver_desc = desc;
                         statement -> ird -> connection = connection;
@@ -859,6 +879,7 @@ SQLRETURN __SQLAllocHandle( SQLSMALLINT handle_type,
                         }
                         statement -> implicit_ipd = statement -> ipd;
                         statement -> ipd -> implicit = 1;
+                        statement -> ipd -> associated_with = statement;
                         statement -> ipd -> state = STATE_D1i;
                         statement -> ipd -> driver_desc = desc;
                         statement -> ipd -> connection = connection;
@@ -867,7 +888,7 @@ SQLRETURN __SQLAllocHandle( SQLSMALLINT handle_type,
                 /* Driver may only have unicode API's */
                 else if ( CHECK_SQLGETSTMTATTRW( connection ))
                 {
-                    SQLHDESC desc;
+                    DRV_SQLHDESC desc;
 
                     /*
                      * ARD
@@ -905,6 +926,7 @@ SQLRETURN __SQLAllocHandle( SQLSMALLINT handle_type,
                         }
                         statement -> implicit_ard = statement -> ard;
                         statement -> ard -> implicit = 1;
+                        statement -> ard -> associated_with = statement;
                         statement -> ard -> state = STATE_D1i;
                         statement -> ard -> driver_desc = desc;
                         statement -> ard -> connection = connection;
@@ -928,7 +950,7 @@ SQLRETURN __SQLAllocHandle( SQLSMALLINT handle_type,
                          * to wrap around this
                          */
                         statement -> apd = __alloc_desc();
-                        if ( !statement -> ard )
+                        if ( !statement -> apd )
                         {
                             dm_log_write( __FILE__, 
                                 __LINE__, 
@@ -948,6 +970,7 @@ SQLRETURN __SQLAllocHandle( SQLSMALLINT handle_type,
                         }
                         statement -> implicit_apd = statement -> apd;
                         statement -> apd -> implicit = 1;
+                        statement -> apd -> associated_with = statement;
                         statement -> apd -> state = STATE_D1i;
                         statement -> apd -> driver_desc = desc;
                         statement -> apd -> connection = connection;
@@ -991,6 +1014,7 @@ SQLRETURN __SQLAllocHandle( SQLSMALLINT handle_type,
                         }
                         statement -> implicit_ird = statement -> ird;
                         statement -> ird -> implicit = 1;
+                        statement -> ird -> associated_with = statement;
                         statement -> ird -> state = STATE_D1i;
                         statement -> ird -> driver_desc = desc;
                         statement -> ird -> connection = connection;
@@ -1032,6 +1056,7 @@ SQLRETURN __SQLAllocHandle( SQLSMALLINT handle_type,
                         }
                         statement -> implicit_ipd = statement -> ipd;
                         statement -> ipd -> implicit = 1;
+                        statement -> ipd -> associated_with = statement;
                         statement -> ipd -> state = STATE_D1i;
                         statement -> ipd -> driver_desc = desc;
                         statement -> ipd -> connection = connection;
@@ -1201,6 +1226,7 @@ SQLRETURN __SQLAllocHandle( SQLSMALLINT handle_type,
 
             descriptor -> state = STATE_D1e;
             descriptor -> implicit = 0;
+            descriptor -> associated_with = NULL;
 
             connection -> statement_count ++;
 
