@@ -27,9 +27,18 @@
  *
  **********************************************************************
  *
- * $Id: SQLBrowseConnect.c,v 1.12 2005/11/21 17:25:43 lurcher Exp $
+ * $Id: SQLBrowseConnect.c,v 1.15 2009/02/18 17:59:08 lurcher Exp $
  *
  * $Log: SQLBrowseConnect.c,v $
+ * Revision 1.15  2009/02/18 17:59:08  lurcher
+ * Shift to using config.h, the compile lines were making it hard to spot warnings
+ *
+ * Revision 1.14  2009/02/17 09:47:44  lurcher
+ * Clear up a number of bugs
+ *
+ * Revision 1.13  2007/10/19 10:14:05  lurcher
+ * Pull errors from SQLBrowseConnect when it returns SQL_NEED_DATA
+ *
  * Revision 1.12  2005/11/21 17:25:43  lurcher
  * A few DM fixes for Oracle's ODBC driver
  *
@@ -162,9 +171,10 @@
  *
  **********************************************************************/
 
+#include <config.h>
 #include "drivermanager.h"
 
-static char const rcsid[]= "$RCSfile: SQLBrowseConnect.c,v $ $Revision: 1.12 $";
+static char const rcsid[]= "$RCSfile: SQLBrowseConnect.c,v $ $Revision: 1.15 $";
 
 #define BUFFER_LEN      4095
 
@@ -466,45 +476,23 @@ SQLRETURN SQLBrowseConnect(
           return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
     } 
 
-    if ( ret == SQL_NEED_DATA )
-    {
-        connection -> state = STATE_C3;
-
-        if ( log_info.log_flag )
-        {
-            sprintf( connection -> msg, 
-                    "\n\t\tExit:[%s]\
-                	\n\t\t\tPtr Conn Str Out = %s",
-                    __get_return_status( ret, s1 ),
-                    __sptr_as_string( s2, ptr_conn_str_out ));
-
-            dm_log_write( __FILE__, 
-                    __LINE__, 
-                    LOG_INFO, 
-                    LOG_INFO, 
-                    connection -> msg );
-        }
-
-        return function_return( SQL_HANDLE_DBC, connection, ret );
-    }
-    else if ( !SQL_SUCCEEDED( ret ))
+    if ( !SQL_SUCCEEDED( ret ) || ret == SQL_NEED_DATA )
     {
         SQLCHAR sqlstate[ 6 ];
         SQLINTEGER native_error;
         SQLSMALLINT ind;
         SQLCHAR message_text[ SQL_MAX_MESSAGE_LENGTH + 1 ];
-        SQLRETURN ret;
-        char buf[ 128 ];
+        SQLRETURN eret;
 
         /*
          * get the error from the driver before
-         * loseing the connection
+         * looseing the connection
          */
         if ( CHECK_SQLERROR( connection ))
         {
             do
             {
-                ret = SQLERROR( connection,
+                eret = SQLERROR( connection,
                     SQL_NULL_HENV,
                         connection -> driver_dbc,
                         SQL_NULL_HSTMT,
@@ -515,7 +503,7 @@ SQLRETURN SQLBrowseConnect(
                         &ind );
 
 
-                if ( SQL_SUCCEEDED( ret ))
+                if ( SQL_SUCCEEDED( eret ))
                 {
                     __post_internal_error_ex( &connection -> error,
                             sqlstate,
@@ -529,7 +517,7 @@ SQLRETURN SQLBrowseConnect(
                     dm_log_write_diag( connection -> msg );
                 }
             }
-            while( SQL_SUCCEEDED( ret ));
+            while( SQL_SUCCEEDED( eret ));
         }
         else if ( CHECK_SQLGETDIAGREC( connection ))
         {
@@ -537,7 +525,7 @@ SQLRETURN SQLBrowseConnect(
 
             do
             {
-                ret = SQLGETDIAGREC( connection,
+                eret = SQLGETDIAGREC( connection,
                         SQL_HANDLE_DBC,
                         connection -> driver_dbc,
                         rec ++,
@@ -547,7 +535,7 @@ SQLRETURN SQLBrowseConnect(
                         sizeof( message_text ),
                         &ind );
 
-                if ( SQL_SUCCEEDED( ret ))
+                if ( SQL_SUCCEEDED( eret ))
                 {
                     __post_internal_error_ex( &connection -> error,
                             sqlstate,
@@ -561,12 +549,18 @@ SQLRETURN SQLBrowseConnect(
                     dm_log_write_diag( connection -> msg );
                 }
             }
-            while( SQL_SUCCEEDED( ret ));
+            while( SQL_SUCCEEDED( eret ));
         }
 
-        __disconnect_part_one( connection );
-
-        connection -> state = STATE_C2;
+    	if ( ret != SQL_NEED_DATA ) 
+		{
+        	__disconnect_part_one( connection );
+        	connection -> state = STATE_C2;
+		}
+		else 
+		{
+       		connection -> state = STATE_C3;
+		}
     }
     else
     {
