@@ -27,9 +27,21 @@
  *
  **********************************************************************
  *
- * $Id: SQLColAttribute.c,v 1.15 2006/03/08 09:18:41 lurcher Exp $
+ * $Id: SQLColAttribute.c,v 1.19 2009/02/18 17:59:08 lurcher Exp $
  *
  * $Log: SQLColAttribute.c,v $
+ * Revision 1.19  2009/02/18 17:59:08  lurcher
+ * Shift to using config.h, the compile lines were making it hard to spot warnings
+ *
+ * Revision 1.18  2009/02/17 09:47:44  lurcher
+ * Clear up a number of bugs
+ *
+ * Revision 1.17  2008/09/29 14:02:43  lurcher
+ * Fix missing dlfcn group option
+ *
+ * Revision 1.16  2007/04/02 10:50:17  lurcher
+ * Fix some 64bit problems (only when sizeof(SQLLEN) == 8 )
+ *
  * Revision 1.15  2006/03/08 09:18:41  lurcher
  * fix silly typo that was using sizeof( SQL_WCHAR ) instead of SQLWCHAR
  *
@@ -170,9 +182,10 @@
  *
  **********************************************************************/
 
+#include <config.h>
 #include "drivermanager.h"
 
-static char const rcsid[]= "$RCSfile: SQLColAttribute.c,v $ $Revision: 1.15 $";
+static char const rcsid[]= "$RCSfile: SQLColAttribute.c,v $ $Revision: 1.19 $";
 
 SQLINTEGER map_ca_odbc3_to_2( SQLINTEGER field_identifier )
 {
@@ -219,7 +232,7 @@ SQLRETURN SQLColAttributeA( SQLHSTMT statement_handle,
            SQLPOINTER character_attribute,
            SQLSMALLINT buffer_length,
            SQLSMALLINT *string_length,
-           SQLPOINTER numeric_attribute )
+           SQLLEN *numeric_attribute )
 {
     return SQLColAttribute( statement_handle,
                             (SQLUSMALLINT) column_number,
@@ -236,7 +249,7 @@ SQLRETURN SQLColAttribute ( SQLHSTMT statement_handle,
            SQLPOINTER character_attribute,
            SQLSMALLINT buffer_length,
            SQLSMALLINT *string_length,
-           SQLPOINTER numeric_attribute )
+           SQLLEN *numeric_attribute )
 {
     DMHSTMT statement = (DMHSTMT) statement_handle;
     SQLRETURN ret = 0;
@@ -261,13 +274,13 @@ SQLRETURN SQLColAttribute ( SQLHSTMT statement_handle,
     if ( log_info.log_flag )
     {
         sprintf( statement -> msg, "\n\t\tEntry:\
-            \n\t\t\tStatement = %p\
-            \n\t\t\tColumn Number = %d\
-            \n\t\t\tField Identifier = %s\
-            \n\t\t\tCharacter Attr = %p\
-            \n\t\t\tBuffer Length = %d\
-            \n\t\t\tString Length = %p\
-            \n\t\t\tNumeric Attribute = %p",
+\n\t\t\tStatement = %p\
+\n\t\t\tColumn Number = %d\
+\n\t\t\tField Identifier = %s\
+\n\t\t\tCharacter Attr = %p\
+\n\t\t\tBuffer Length = %d\
+\n\t\t\tString Length = %p\
+\n\t\t\tNumeric Attribute = %p",
                 statement,
                 column_number,
                 __col_attr_as_string( s1, field_identifier ),
@@ -405,7 +418,7 @@ SQLRETURN SQLColAttribute ( SQLHSTMT statement_handle,
     {
         if ( !CHECK_SQLCOLATTRIBUTEW( statement -> connection ))
         {
-            if ( ret = CHECK_SQLCOLATTRIBUTESW( statement -> connection ))
+            if (( ret = CHECK_SQLCOLATTRIBUTESW( statement -> connection )))
             {
                 SQLWCHAR *s1 = NULL;
 
@@ -431,7 +444,7 @@ SQLRETURN SQLColAttribute ( SQLHSTMT statement_handle,
                   case SQL_DESC_NAME:
                     if ( SQL_SUCCEEDED( ret ) && character_attribute && buffer_length > 0 )
                     {
-                        s1 = malloc( sizeof( SQLWCHAR ) * ( buffer_length + 1 ));
+                        s1 = calloc( sizeof( SQLWCHAR ) * ( buffer_length + 1 ), 1);
                     }
                     break;
 
@@ -464,7 +477,7 @@ SQLRETURN SQLColAttribute ( SQLHSTMT statement_handle,
                   case SQL_DESC_NAME:
                     if ( SQL_SUCCEEDED( ret ) && character_attribute && s1 )
                     {
-                        unicode_to_ansi_copy( character_attribute, s1,  SQL_NTS, statement -> connection );
+                        unicode_to_ansi_copy( character_attribute, buffer_length, s1,  SQL_NTS, statement -> connection );
                     }
 					if ( SQL_SUCCEEDED( ret ) && string_length ) 
 					{
@@ -516,7 +529,7 @@ SQLRETURN SQLColAttribute ( SQLHSTMT statement_handle,
               case SQL_COLUMN_NAME:
                 if ( SQL_SUCCEEDED( ret ) && character_attribute && buffer_length > 0 )
                 {
-                    s1 = malloc( sizeof( SQLWCHAR ) * ( buffer_length + 1 ));
+                    s1 = calloc( sizeof( SQLWCHAR ) * ( buffer_length + 1 ), 1);
                 }
                 break;
 
@@ -549,7 +562,7 @@ SQLRETURN SQLColAttribute ( SQLHSTMT statement_handle,
               case SQL_COLUMN_NAME:
                 if ( SQL_SUCCEEDED( ret ) && character_attribute && s1 )
                 {
-                    unicode_to_ansi_copy( character_attribute, s1, SQL_NTS, statement -> connection );
+                    unicode_to_ansi_copy( character_attribute, buffer_length, s1, SQL_NTS, statement -> connection );
                 }
 				if ( SQL_SUCCEEDED( ret ) && string_length ) 
 				{
