@@ -27,9 +27,15 @@
  *
  **********************************************************************
  *
- * $Id: SQLColAttributes.c,v 1.13 2006/03/08 09:18:41 lurcher Exp $
+ * $Id: SQLColAttributes.c,v 1.15 2009/02/18 17:59:08 lurcher Exp $
  *
  * $Log: SQLColAttributes.c,v $
+ * Revision 1.15  2009/02/18 17:59:08  lurcher
+ * Shift to using config.h, the compile lines were making it hard to spot warnings
+ *
+ * Revision 1.14  2008/09/29 14:02:43  lurcher
+ * Fix missing dlfcn group option
+ *
  * Revision 1.13  2006/03/08 09:18:41  lurcher
  * fix silly typo that was using sizeof( SQL_WCHAR ) instead of SQLWCHAR
  *
@@ -161,9 +167,10 @@
  *
  **********************************************************************/
 
+#include <config.h>
 #include "drivermanager.h"
 
-static char const rcsid[]= "$RCSfile: SQLColAttributes.c,v $ $Revision: 1.13 $";
+static char const rcsid[]= "$RCSfile: SQLColAttributes.c,v $ $Revision: 1.15 $";
 
 SQLINTEGER  map_ca_odbc2_to_3( SQLINTEGER field_identifier )
 {
@@ -254,13 +261,13 @@ SQLRETURN SQLColAttributes( SQLHSTMT statement_handle,
     if ( log_info.log_flag )
     {
         sprintf( statement -> msg, "\n\t\tEntry:\
-            \n\t\t\tStatement = %p\
-            \n\t\t\tColumn Number = %d\
-            \n\t\t\tField Identifier = %s\
-            \n\t\t\tCharacter Attr = %p\
-            \n\t\t\tBuffer Length = %d\
-            \n\t\t\tString Length = %p\
-            \n\t\t\tNumeric Attribute = %p",
+\n\t\t\tStatement = %p\
+\n\t\t\tColumn Number = %d\
+\n\t\t\tField Identifier = %s\
+\n\t\t\tCharacter Attr = %p\
+\n\t\t\tBuffer Length = %d\
+\n\t\t\tString Length = %p\
+\n\t\t\tNumeric Attribute = %p",
                 statement,
                 column_number,
                 __col_attr_as_string( s1, field_identifier ),
@@ -416,7 +423,7 @@ SQLRETURN SQLColAttributes( SQLHSTMT statement_handle,
 
                 if ( character_attribute && buffer_length > 0 )
                 {
-                    s1 = malloc( sizeof( SQLWCHAR ) * ( buffer_length + 1 ));
+                    s1 = calloc( sizeof( SQLWCHAR ) * ( buffer_length + 1 ), 1);
                 }
 
                 ret = SQLCOLATTRIBUTEW( statement -> connection,
@@ -430,7 +437,7 @@ SQLRETURN SQLColAttributes( SQLHSTMT statement_handle,
 
                 if ( SQL_SUCCEEDED( ret ) && character_attribute && s1 )
                 {
-                    unicode_to_ansi_copy( character_attribute, s1, SQL_NTS, statement -> connection );
+                    unicode_to_ansi_copy( character_attribute, buffer_length, s1, SQL_NTS, statement -> connection );
                 }
 				if ( SQL_SUCCEEDED( ret ) && string_length && character_attribute ) 
 				{
@@ -463,7 +470,7 @@ SQLRETURN SQLColAttributes( SQLHSTMT statement_handle,
 
             if ( character_attribute && buffer_length > 0 )
             {
-                s1 = malloc( sizeof( SQLWCHAR ) * ( buffer_length + 1 ));
+                s1 = calloc( sizeof( SQLWCHAR ) * ( buffer_length + 1 ), 1);
             }
 
             ret = SQLCOLATTRIBUTESW( statement -> connection,
@@ -477,7 +484,7 @@ SQLRETURN SQLColAttributes( SQLHSTMT statement_handle,
 
             if ( SQL_SUCCEEDED( ret ) && character_attribute )
             {
-                unicode_to_ansi_copy( character_attribute, s1, SQL_NTS, statement -> connection );
+                unicode_to_ansi_copy( character_attribute, buffer_length, s1, SQL_NTS, statement -> connection );
 			}
 			if ( SQL_SUCCEEDED( ret ) && string_length && character_attribute ) 
 			{
