@@ -27,9 +27,15 @@
  *
  **********************************************************************
  *
- * $Id: SQLColumns.c,v 1.6 2004/01/12 09:54:39 lurcher Exp $
+ * $Id: SQLColumns.c,v 1.8 2009/02/18 17:59:08 lurcher Exp $
  *
  * $Log: SQLColumns.c,v $
+ * Revision 1.8  2009/02/18 17:59:08  lurcher
+ * Shift to using config.h, the compile lines were making it hard to spot warnings
+ *
+ * Revision 1.7  2008/08/29 08:01:38  lurcher
+ * Alter the way W functions are passed to the driver
+ *
  * Revision 1.6  2004/01/12 09:54:39  lurcher
  *
  * Fix problem where STATE_S5 stops metadata calls
@@ -135,9 +141,10 @@
  *
  **********************************************************************/
 
+#include <config.h>
 #include "drivermanager.h"
 
-static char const rcsid[]= "$RCSfile: SQLColumns.c,v $ $Revision: 1.6 $";
+static char const rcsid[]= "$RCSfile: SQLColumns.c,v $ $Revision: 1.8 $";
 
 SQLRETURN SQLColumnsA( SQLHSTMT statement_handle,
            SQLCHAR *catalog_name,
@@ -193,11 +200,11 @@ SQLRETURN SQLColumns( SQLHSTMT statement_handle,
     if ( log_info.log_flag )
     {
         sprintf( statement -> msg, "\n\t\tEntry:\
-            \n\t\t\tStatement = %p\
-            \n\t\t\tCatalog Name = %s\
-            \n\t\t\tSchema Name = %s\
-            \n\t\t\tTable Name = %s\
-            \n\t\t\tColumn Type = %s",
+\n\t\t\tStatement = %p\
+\n\t\t\tCatalog Name = %s\
+\n\t\t\tSchema Name = %s\
+\n\t\t\tTable Name = %s\
+\n\t\t\tColumn Name = %s",
                 statement,
                 __string_with_length( s1, catalog_name, name_length1 ),
                 __string_with_length( s2, schema_name, name_length2 ),
@@ -234,7 +241,7 @@ SQLRETURN SQLColumns( SQLHSTMT statement_handle,
             statement -> state == STATE_S6 ||
             statement -> state == STATE_S7 )
 #else
-    if ( statement -> state == STATE_S6 ||
+    if (( statement -> state == STATE_S6 && statement -> eod == 0 ) ||
             statement -> state == STATE_S7 )
 #endif
     {
