@@ -27,9 +27,15 @@
  *
  **********************************************************************
  *
- * $Id: SQLDataSources.c,v 1.7 2003/10/30 18:20:45 lurcher Exp $
+ * $Id: SQLDataSources.c,v 1.9 2009/02/18 17:59:08 lurcher Exp $
  *
  * $Log: SQLDataSources.c,v $
+ * Revision 1.9  2009/02/18 17:59:08  lurcher
+ * Shift to using config.h, the compile lines were making it hard to spot warnings
+ *
+ * Revision 1.8  2008/06/30 08:40:48  lurcher
+ * Few more tweeks towards a release
+ *
  * Revision 1.7  2003/10/30 18:20:45  lurcher
  *
  * Fix broken thread protection
@@ -126,11 +132,12 @@
  *
  **********************************************************************/
 
+#include <config.h>
 #include "drivermanager.h"
 
-static char const rcsid[]= "$RCSfile: SQLDataSources.c,v $ $Revision: 1.7 $";
+static char const rcsid[]= "$RCSfile: SQLDataSources.c,v $ $Revision: 1.9 $";
 
-#define BUFFERSIZE      1024
+#define BUFFERSIZE      1024*4
 
 SQLRETURN SQLDataSourcesA( SQLHENV environment_handle,
            SQLUSMALLINT direction,
@@ -184,7 +191,7 @@ SQLRETURN SQLDataSources( SQLHENV environment_handle,
     if ( log_info.log_flag )
     {
         sprintf( environment -> msg, "\n\t\tEntry:\
-            \n\t\t\tEnvironment = %p",
+\n\t\t\tEnvironment = %p",
                 environment );
 
         dm_log_write( __FILE__, 
