@@ -26,9 +26,15 @@
  *
  **********************************************************************
  *
- * $Id: SQLExecDirect.c,v 1.9 2003/10/30 18:20:45 lurcher Exp $
+ * $Id: SQLExecDirect.c,v 1.11 2009/02/18 17:59:08 lurcher Exp $
  *
  * $Log: SQLExecDirect.c,v $
+ * Revision 1.11  2009/02/18 17:59:08  lurcher
+ * Shift to using config.h, the compile lines were making it hard to spot warnings
+ *
+ * Revision 1.10  2006/04/11 10:22:56  lurcher
+ * Fix a data type check
+ *
  * Revision 1.9  2003/10/30 18:20:45  lurcher
  *
  * Fix broken thread protection
@@ -163,9 +169,10 @@
  *
  **********************************************************************/
 
+#include <config.h>
 #include "drivermanager.h"
 
-static char const rcsid[]= "$RCSfile: SQLExecDirect.c,v $ $Revision: 1.9 $";
+static char const rcsid[]= "$RCSfile: SQLExecDirect.c,v $ $Revision: 1.11 $";
 
 SQLRETURN SQLExecDirectA( SQLHSTMT statement_handle,
            SQLCHAR *statement_text,
@@ -210,20 +217,20 @@ SQLRETURN SQLExecDirect( SQLHSTMT statement_handle,
 
         if ( statement_text && text_length == SQL_NTS )
         {
-            s1 = malloc( strlen((char*) statement_text ) + 101 );
+            s1 = malloc( strlen((char*) statement_text ) + LOG_MESSAGE_LEN );
         }
         else if ( statement_text )
         {
-            s1 = malloc( text_length + 101 );
+            s1 = malloc( text_length + LOG_MESSAGE_LEN );
         }
         else
         {
-            s1 = malloc( 101 );
+            s1 = malloc( LOG_MESSAGE_LEN );
         }
 
         sprintf( statement -> msg, "\n\t\tEntry:\
-            \n\t\t\tStatement = %p\
-            \n\t\t\tSQL = %s",
+\n\t\t\tStatement = %p\
+\n\t\t\tSQL = %s",
                 statement,
                 __string_with_length( s1, statement_text, text_length ));
 
@@ -277,7 +284,7 @@ SQLRETURN SQLExecDirect( SQLHSTMT statement_handle,
             statement -> state == STATE_S6 ||
             statement -> state == STATE_S7 )
 #else
-    if ( statement -> state == STATE_S6 ||
+    if (( statement -> state == STATE_S6 && statement -> eod == 0 ) ||
             statement -> state == STATE_S7 )
 #endif
     {
