@@ -27,9 +27,21 @@
  *
  **********************************************************************
  *
- * $Id: SQLFreeHandle.c,v 1.8 2006/04/18 10:24:47 lurcher Exp $
+ * $Id: SQLFreeHandle.c,v 1.12 2009/02/18 17:59:08 lurcher Exp $
  *
  * $Log: SQLFreeHandle.c,v $
+ * Revision 1.12  2009/02/18 17:59:08  lurcher
+ * Shift to using config.h, the compile lines were making it hard to spot warnings
+ *
+ * Revision 1.11  2009/02/17 09:47:44  lurcher
+ * Clear up a number of bugs
+ *
+ * Revision 1.10  2007/12/17 13:13:03  lurcher
+ * Fix a couple of descriptor typo's
+ *
+ * Revision 1.9  2007/01/02 10:27:50  lurcher
+ * Fix descriptor leak with unicode only driver
+ *
  * Revision 1.8  2006/04/18 10:24:47  lurcher
  * Add a couple of changes from Mark Vanderwiel
  *
@@ -161,13 +173,14 @@
  *
  **********************************************************************/
 
+#include <config.h>
 #include "drivermanager.h"
 #if defined ( COLLECT_STATS ) && defined( HAVE_SYS_SEM_H )
 #include "__stats.h"
 #include <uodbc_stats.h>
 #endif
 
-static char const rcsid[]= "$RCSfile: SQLFreeHandle.c,v $ $Revision: 1.8 $";
+static char const rcsid[]= "$RCSfile: SQLFreeHandle.c,v $ $Revision: 1.12 $";
 
 SQLRETURN __SQLFreeHandle( SQLSMALLINT handle_type,
         SQLHANDLE handle )
@@ -434,9 +447,22 @@ SQLRETURN __SQLFreeHandle( SQLSMALLINT handle_type,
             if ( SQL_SUCCEEDED( ret ))
             {
                 /*
-                 * release the implicit descriptors
+                 * break any association
                  */
-                if ( statement -> connection -> driver_act_ver == 3 )
+
+                if ( statement -> ard ) {
+                    statement -> ard -> associated_with = NULL;
+                }
+                if ( statement -> apd ) {
+                    statement -> apd -> associated_with = NULL;
+                }
+                /*
+                 * release the implicit descriptors, 
+				 * this matches the tests in SQLAllocHandle
+                 */
+                if (( statement -> connection -> driver_act_ver == 3 &&
+						CHECK_SQLGETSTMTATTR( connection )) ||
+						CHECK_SQLGETSTMTATTRW( connection ))
                 {
                     if ( statement -> implicit_ard )
                         __release_desc( statement -> implicit_ard );
@@ -483,7 +509,6 @@ SQLRETURN __SQLFreeHandle( SQLSMALLINT handle_type,
             DMHDESC descriptor = (DMHDESC)handle;
             DMHDBC connection;
             SQLRETURN ret;
-            SQLCHAR s0[ 20 ];
 
             /*
              * check descriptor
@@ -511,6 +536,21 @@ SQLRETURN __SQLFreeHandle( SQLSMALLINT handle_type,
                         descriptor -> msg );
             }
 
+			if ( descriptor -> implicit )
+			{
+				dm_log_write( __FILE__,
+						__LINE__,
+						LOG_INFO,
+						LOG_INFO,
+						"Error: HY017" );
+		
+				__post_internal_error( &descriptor -> error,
+						ERROR_HY017, NULL,
+						connection -> environment -> requested_version );
+		
+				return function_return( SQL_HANDLE_DESC, descriptor, SQL_ERROR );
+			}
+		
             thread_protect( SQL_HANDLE_DESC, descriptor );
 
             if ( !CHECK_SQLFREEHANDLE( connection ))
@@ -536,6 +576,29 @@ SQLRETURN __SQLFreeHandle( SQLSMALLINT handle_type,
                         descriptor -> driver_desc );
             }
 
+            /*
+             * check status of statements associated with this descriptor
+             */
+
+            if( __check_stmt_from_desc( descriptor, STATE_S8 ) ||
+                __check_stmt_from_desc( descriptor, STATE_S9 ) ||
+                __check_stmt_from_desc( descriptor, STATE_S10 ) ||
+                __check_stmt_from_desc( descriptor, STATE_S11 ) ||
+                __check_stmt_from_desc( descriptor, STATE_S12 )) {
+
+                dm_log_write( __FILE__, 
+                        __LINE__, 
+                        LOG_INFO, 
+                        LOG_INFO, 
+                        "Error: HY010" );
+
+                __post_internal_error( &descriptor -> error,
+                        ERROR_HY010, NULL,
+                        descriptor -> connection -> environment -> requested_version );
+
+                return function_return( SQL_HANDLE_DESC, descriptor, SQL_ERROR );
+            }
+
             thread_release( SQL_HANDLE_DESC, descriptor );
 
             __release_desc( descriptor );
