@@ -27,9 +27,27 @@
  *
  **********************************************************************
  *
- * $Id: SQLGetDiagField.c,v 1.11 2006/05/31 17:35:34 lurcher Exp $
+ * $Id: SQLGetDiagField.c,v 1.17 2009/02/18 17:59:08 lurcher Exp $
  *
  * $Log: SQLGetDiagField.c,v $
+ * Revision 1.17  2009/02/18 17:59:08  lurcher
+ * Shift to using config.h, the compile lines were making it hard to spot warnings
+ *
+ * Revision 1.16  2009/02/17 09:47:44  lurcher
+ * Clear up a number of bugs
+ *
+ * Revision 1.15  2009/02/04 09:30:01  lurcher
+ * Fix some SQLINTEGER/SQLLEN conflicts
+ *
+ * Revision 1.14  2008/09/29 14:02:45  lurcher
+ * Fix missing dlfcn group option
+ *
+ * Revision 1.13  2007/03/05 09:49:24  lurcher
+ * Get it to build on VMS again
+ *
+ * Revision 1.12  2006/11/27 14:08:33  lurcher
+ * Sync up dirs
+ *
  * Revision 1.11  2006/05/31 17:35:34  lurcher
  * Add unicode ODBCINST entry points
  *
@@ -176,15 +194,18 @@
  *
  **********************************************************************/
 
+#include <config.h>
 #include "drivermanager.h"
 
-static char const rcsid[]= "$RCSfile: SQLGetDiagField.c,v $ $Revision: 1.11 $";
+static char const rcsid[]= "$RCSfile: SQLGetDiagField.c,v $ $Revision: 1.17 $";
 
 #define ODBC30_SUBCLASS        "01S00,01S01,01S02,01S06,01S07,07S01,08S01,21S01,\
 21S02,25S01,25S02,25S03,42S01,42S02,42S11,42S12,42S21,42S22,HY095,HY097,HY098,\
 HY099,HY100,HY101,HY105,HY107,HY109,HY110,HY111,HYT00,HYT01,IM001,IM002,IM003,\
 IM004,IM005,IM006,IM007,IM008,IM010,IM011,IM012"
 
+extern int __is_env( EHEAD * head );
+
 static SQLRETURN extract_sql_error_field( EHEAD *head,
                 SQLSMALLINT rec_number,
                 SQLSMALLINT diag_identifier,
@@ -292,7 +313,7 @@ static SQLRETURN extract_sql_error_field( EHEAD *head,
             {
                 if ( SQL_SUCCEEDED( head -> diag_dynamic_function_ret ) && diag_info_ptr )
                 {
-                    unicode_to_ansi_copy( diag_info_ptr, head -> diag_dynamic_function, buffer_length, __get_connection( head ));
+                    unicode_to_ansi_copy( diag_info_ptr, buffer_length, head -> diag_dynamic_function, buffer_length, __get_connection( head ));
                     if ( string_length_ptr )
                     {
                         *string_length_ptr = wide_strlen( head -> diag_dynamic_function );
@@ -303,7 +324,7 @@ static SQLRETURN extract_sql_error_field( EHEAD *head,
             else if ( __get_connection( head ) -> unicode_driver &&
                 CHECK_SQLGETDIAGFIELDW( __get_connection( head )))
             {
-                SQLWCHAR *s1;
+                SQLWCHAR *s1 = NULL;
 
                 if ( buffer_length > 0 )
                 {
@@ -321,7 +342,7 @@ static SQLRETURN extract_sql_error_field( EHEAD *head,
 
                 if ( SQL_SUCCEEDED( ret ) && diag_info_ptr && s1 )
                 {
-                    unicode_to_ansi_copy( diag_info_ptr, s1, buffer_length, __get_connection( head ));
+                    unicode_to_ansi_copy( diag_info_ptr, buffer_length, s1, buffer_length, __get_connection( head ));
                 }
 
                 if ( s1 )
@@ -455,18 +476,14 @@ static SQLRETURN extract_sql_error_field( EHEAD *head,
             ptr = ptr -> next;
             rec_number --;
         }
+		if ( !ptr ) 
+		{
+			return SQL_NO_DATA;
+		}
     }
     else if ( !__is_env( head ) && __get_connection( head ) -> state != STATE_C2 )
     {
-    	if ( rec_number <= head -> sql_diag_head.internal_count + 
-            head -> sql_diag_head.error_count )
-		{
-        	rec_number -= head -> sql_diag_head.internal_count;
-		}
-		else
-		{
-        	rec_number -= ( head -> sql_diag_head.internal_count + head -> sql_diag_head.error_count );
-		}
+		rec_number -= head -> sql_diag_head.internal_count;
 
         if ( __get_connection( head ) -> unicode_driver &&
             CHECK_SQLGETDIAGFIELDW( __get_connection( head )))
@@ -491,7 +508,7 @@ static SQLRETURN extract_sql_error_field( EHEAD *head,
 
             if ( SQL_SUCCEEDED( ret ) && s1 && diag_info_ptr )
             {
-                unicode_to_ansi_copy( diag_info_ptr, s1, SQL_NTS, __get_connection( head ));
+                unicode_to_ansi_copy( diag_info_ptr, buffer_length, s1, SQL_NTS, __get_connection( head ));
             }
 
             if ( s1 )
@@ -554,6 +571,10 @@ static SQLRETURN extract_sql_error_field( EHEAD *head,
                 ptr = ptr -> next;
                 rec_number --;
             }
+			if ( !ptr ) 
+			{
+	    		return SQL_NO_DATA;
+			}
         }
     }
     else 
@@ -572,7 +593,7 @@ static SQLRETURN extract_sql_error_field( EHEAD *head,
         {
             if ( SQL_SUCCEEDED( ptr -> diag_class_origin_ret ))
             {
-                unicode_to_ansi_copy( diag_info_ptr, ptr -> diag_class_origin, buffer_length, __get_connection( head ));
+                unicode_to_ansi_copy( diag_info_ptr, buffer_length, ptr -> diag_class_origin, buffer_length, __get_connection( head ));
                 if ( string_length_ptr )
                 {
                     *string_length_ptr = wide_strlen( ptr -> diag_class_origin );
@@ -600,7 +621,7 @@ static SQLRETURN extract_sql_error_field( EHEAD *head,
         {
             if ( SQL_SUCCEEDED( ptr -> diag_connection_name_ret ))
             {
-                unicode_to_ansi_copy( diag_info_ptr, ptr -> diag_connection_name, buffer_length, __get_connection( head ));
+                unicode_to_ansi_copy( diag_info_ptr, buffer_length, ptr -> diag_connection_name, buffer_length, __get_connection( head ));
                 if ( string_length_ptr )
                 {
                     *string_length_ptr = wide_strlen( ptr -> diag_connection_name );
@@ -672,7 +693,7 @@ static SQLRETURN extract_sql_error_field( EHEAD *head,
         {
             if ( SQL_SUCCEEDED( ptr -> diag_server_name_ret ))
             {
-                unicode_to_ansi_copy( diag_info_ptr, ptr -> diag_server_name, buffer_length, __get_connection( head ));
+                unicode_to_ansi_copy( diag_info_ptr, buffer_length, ptr -> diag_server_name, buffer_length, __get_connection( head ));
                 if ( string_length_ptr )
                 {
                     *string_length_ptr = wide_strlen( ptr -> diag_server_name );
@@ -731,7 +752,7 @@ static SQLRETURN extract_sql_error_field( EHEAD *head,
         {
             if ( SQL_SUCCEEDED( ptr -> diag_subclass_origin_ret ))
             {
-                unicode_to_ansi_copy( diag_info_ptr, ptr -> diag_subclass_origin, buffer_length, __get_connection( head ));
+                unicode_to_ansi_copy( diag_info_ptr, buffer_length, ptr -> diag_subclass_origin, buffer_length, __get_connection( head ));
                 if ( string_length_ptr )
                 {
                     *string_length_ptr = wide_strlen( ptr -> diag_subclass_origin );
@@ -798,12 +819,12 @@ SQLRETURN SQLGetDiagField( SQLSMALLINT handle_type,
         {
             sprintf( environment -> msg, 
                 "\n\t\tEntry:\
-                \n\t\t\tEnvironment = %p\
-                \n\t\t\tRec Number = %d\
-                \n\t\t\tDiag Ident = %d\
-                \n\t\t\tDiag Info Ptr = %p\
-                \n\t\t\tBuffer Length = %d\
-                \n\t\t\tString Len Ptr = %p",
+\n\t\t\tEnvironment = %p\
+\n\t\t\tRec Number = %d\
+\n\t\t\tDiag Ident = %d\
+\n\t\t\tDiag Info Ptr = %p\
+\n\t\t\tBuffer Length = %d\
+\n\t\t\tString Len Ptr = %p",
                     environment,
                     rec_number,
                     diag_identifier,
@@ -857,12 +878,12 @@ SQLRETURN SQLGetDiagField( SQLSMALLINT handle_type,
         {
             sprintf( connection -> msg, 
                 "\n\t\tEntry:\
-                \n\t\t\tConnection = %p\
-                \n\t\t\tRec Number = %d\
-                \n\t\t\tDiag Ident = %d\
-                \n\t\t\tDiag Info Ptr = %p\
-                \n\t\t\tBuffer Length = %d\
-                \n\t\t\tString Len Ptr = %p",
+\n\t\t\tConnection = %p\
+\n\t\t\tRec Number = %d\
+\n\t\t\tDiag Ident = %d\
+\n\t\t\tDiag Info Ptr = %p\
+\n\t\t\tBuffer Length = %d\
+\n\t\t\tString Len Ptr = %p",
                     connection,
                     rec_number,
                     diag_identifier,
@@ -916,12 +937,12 @@ SQLRETURN SQLGetDiagField( SQLSMALLINT handle_type,
         {
             sprintf( statement -> msg, 
                 "\n\t\tEntry:\
-                \n\t\t\tStatement = %p\
-                \n\t\t\tRec Number = %d\
-                \n\t\t\tDiag Ident = %d\
-                \n\t\t\tDiag Info Ptr = %p\
-                \n\t\t\tBuffer Length = %d\
-                \n\t\t\tString Len Ptr = %p",
+\n\t\t\tStatement = %p\
+\n\t\t\tRec Number = %d\
+\n\t\t\tDiag Ident = %d\
+\n\t\t\tDiag Info Ptr = %p\
+\n\t\t\tBuffer Length = %d\
+\n\t\t\tString Len Ptr = %p",
                     statement,
                     rec_number,
                     diag_identifier,
@@ -975,12 +996,12 @@ SQLRETURN SQLGetDiagField( SQLSMALLINT handle_type,
         {
             sprintf( descriptor -> msg, 
                 "\n\t\tEntry:\
-                \n\t\t\tDescriptor = %p\
-                \n\t\t\tRec Number = %d\
-                \n\t\t\tDiag Ident = %d\
-                \n\t\t\tDiag Info Ptr = %p\
-                \n\t\t\tBuffer Length = %d\
-                \n\t\t\tString Len Ptr = %p",
+\n\t\t\tDescriptor = %p\
+\n\t\t\tRec Number = %d\
+\n\t\t\tDiag Ident = %d\
+\n\t\t\tDiag Info Ptr = %p\
+\n\t\t\tBuffer Length = %d\
+\n\t\t\tString Len Ptr = %p",
                     descriptor,
                     rec_number,
                     diag_identifier,
