@@ -27,9 +27,30 @@
  *
  **********************************************************************
  *
- * $Id: SQLGetDiagRec.c,v 1.14 2006/05/31 17:35:34 lurcher Exp $
+ * $Id: SQLGetDiagRec.c,v 1.21 2009/02/18 17:59:08 lurcher Exp $
  *
  * $Log: SQLGetDiagRec.c,v $
+ * Revision 1.21  2009/02/18 17:59:08  lurcher
+ * Shift to using config.h, the compile lines were making it hard to spot warnings
+ *
+ * Revision 1.20  2009/02/17 09:47:44  lurcher
+ * Clear up a number of bugs
+ *
+ * Revision 1.19  2009/02/04 09:30:02  lurcher
+ * Fix some SQLINTEGER/SQLLEN conflicts
+ *
+ * Revision 1.18  2008/09/29 14:02:45  lurcher
+ * Fix missing dlfcn group option
+ *
+ * Revision 1.17  2008/05/20 13:43:47  lurcher
+ * Vms fixes
+ *
+ * Revision 1.16  2007/02/12 11:49:34  lurcher
+ * Add QT4 support to existing GUI parts
+ *
+ * Revision 1.15  2006/11/27 14:08:34  lurcher
+ * Sync up dirs
+ *
  * Revision 1.14  2006/05/31 17:35:34  lurcher
  * Add unicode ODBCINST entry points
  *
@@ -159,9 +180,10 @@
  *
  **********************************************************************/
 
+#include <config.h>
 #include "drivermanager.h"
 
-static char const rcsid[]= "$RCSfile: SQLGetDiagRec.c,v $ $Revision: 1.14 $";
+static char const rcsid[]= "$RCSfile: SQLGetDiagRec.c,v $ $Revision: 1.21 $";
 
 int __is_env( EHEAD * head )
 {
@@ -244,7 +266,7 @@ int __get_version( EHEAD * head )
 }
     
 
-SQLHANDLE __get_driver_handle( EHEAD * head )
+DRV_SQLHANDLE __get_driver_handle( EHEAD * head )
 {
     int type;
 
@@ -301,11 +323,16 @@ static SQLRETURN extract_sql_error_rec( EHEAD *head,
             rec_number --;
         }
 
+		if ( !ptr ) 
+		{
+	    	return SQL_NO_DATA;
+		}
+
         as1 = (SQLCHAR*) unicode_to_ansi_alloc( ptr -> msg, SQL_NTS, __get_connection( head ));
 
         if ( sqlstate )
         {
-            unicode_to_ansi_copy((char*) sqlstate, ptr -> sqlstate, SQL_NTS, __get_connection( head ));
+            unicode_to_ansi_copy((char*) sqlstate, 6, ptr -> sqlstate, SQL_NTS, __get_connection( head ));
         }
         if ( buffer_length < strlen((char*) as1 ) + 1 )
         {
@@ -355,18 +382,9 @@ static SQLRETURN extract_sql_error_rec( EHEAD *head,
     else if ( !__is_env( head ) && __get_connection( head ) -> state != STATE_C2 )
     {
         ERROR *ptr;
-        SQLCHAR *as1 = NULL;
         SQLWCHAR *s1 = NULL, *s2 = NULL;
 
-    	if ( rec_number <= head -> sql_diag_head.internal_count + 
-            head -> sql_diag_head.error_count )
-		{
-        	rec_number -= head -> sql_diag_head.internal_count;
-		}
-		else
-		{
-        	rec_number -= ( head -> sql_diag_head.internal_count + head -> sql_diag_head.error_count );
-		}
+		rec_number -= head -> sql_diag_head.internal_count;
 
         s1 = malloc( sizeof( SQLWCHAR ) * ( 6 + 1 ));
 
@@ -396,12 +414,12 @@ static SQLRETURN extract_sql_error_rec( EHEAD *head,
             {
                 if ( sqlstate )
                 {
-                    unicode_to_ansi_copy((char*) sqlstate, s1, SQL_NTS, __get_connection( head ));
+                    unicode_to_ansi_copy((char*) sqlstate, 6, s1, SQL_NTS, __get_connection( head ));
                     __map_error_state((char*) sqlstate, __get_version( head ));
                 }
                 if ( message_text )
                 {
-                    unicode_to_ansi_copy((char*) message_text, s2, SQL_NTS, __get_connection( head ));
+                    unicode_to_ansi_copy((char*) message_text, buffer_length, s2, SQL_NTS, __get_connection( head ));
                 }
             }
 
@@ -437,11 +455,16 @@ static SQLRETURN extract_sql_error_rec( EHEAD *head,
                 rec_number --;
             }
 
+			if ( !ptr ) 
+			{
+	    		return SQL_NO_DATA;
+			}
+
             as1 = (SQLCHAR*) unicode_to_ansi_alloc( ptr -> msg, SQL_NTS, __get_connection( head ));
 
             if ( sqlstate )
             {
-                unicode_to_ansi_copy((char*) sqlstate, ptr -> sqlstate, SQL_NTS, __get_connection( head ));
+                unicode_to_ansi_copy((char*) sqlstate, 6, ptr -> sqlstate, SQL_NTS, __get_connection( head ));
             }
             if ( as1 && buffer_length < strlen((char*) as1 ) + 1 )
             {
@@ -560,13 +583,13 @@ SQLRETURN SQLGetDiagRec( SQLSMALLINT handle_type,
         {
             sprintf( environment -> msg, 
                 "\n\t\tEntry:\
-                \n\t\t\tEnvironment = %p\
-                \n\t\t\tRec Number = %d\
-                \n\t\t\tSQLState = %p\
-                \n\t\t\tNative = %p\
-                \n\t\t\tMessage Text = %p\
-                \n\t\t\tBuffer Length = %d\
-                \n\t\t\tText Len Ptr = %p",
+\n\t\t\tEnvironment = %p\
+\n\t\t\tRec Number = %d\
+\n\t\t\tSQLState = %p\
+\n\t\t\tNative = %p\
+\n\t\t\tMessage Text = %p\
+\n\t\t\tBuffer Length = %d\
+\n\t\t\tText Len Ptr = %p",
                     environment,
                     rec_number,
                     sqlstate,
@@ -596,12 +619,12 @@ SQLRETURN SQLGetDiagRec( SQLSMALLINT handle_type,
             {
                 sprintf( environment -> msg, 
                     "\n\t\tExit:[%s]\
-                    \n\t\t\tSQLState = %s\
-                    \n\t\t\tNative = %s\
-                    \n\t\t\tMessage Text = %s",
+\n\t\t\tSQLState = %s\
+\n\t\t\tNative = %s\
+\n\t\t\tMessage Text = %s",
                         __get_return_status( ret, s2 ),
                         sqlstate,
-                        __ptr_as_string( s0, native ),
+                        __iptr_as_string( s0, native ),
                         __sdata_as_string( s1, SQL_CHAR, 
                             text_length_ptr, message_text ));
             }
@@ -638,13 +661,13 @@ SQLRETURN SQLGetDiagRec( SQLSMALLINT handle_type,
         {
             sprintf( connection -> msg, 
                 "\n\t\tEntry:\
-                \n\t\t\tConnection = %p\
-                \n\t\t\tRec Number = %d\
-                \n\t\t\tSQLState = %p\
-                \n\t\t\tNative = %p\
-                \n\t\t\tMessage Text = %p\
-                \n\t\t\tBuffer Length = %d\
-                \n\t\t\tText Len Ptr = %p",
+\n\t\t\tConnection = %p\
+\n\t\t\tRec Number = %d\
+\n\t\t\tSQLState = %p\
+\n\t\t\tNative = %p\
+\n\t\t\tMessage Text = %p\
+\n\t\t\tBuffer Length = %d\
+\n\t\t\tText Len Ptr = %p",
                     connection,
                     rec_number,
                     sqlstate,
@@ -674,12 +697,12 @@ SQLRETURN SQLGetDiagRec( SQLSMALLINT handle_type,
             {
                 sprintf( connection -> msg, 
                     "\n\t\tExit:[%s]\
-                    \n\t\t\tSQLState = %s\
-                    \n\t\t\tNative = %s\
-                    \n\t\t\tMessage Text = %s",
+\n\t\t\tSQLState = %s\
+\n\t\t\tNative = %s\
+\n\t\t\tMessage Text = %s",
                         __get_return_status( ret, s2 ),
                         sqlstate,
-                        __ptr_as_string( s0, native ),
+                        __iptr_as_string( s0, native ),
                         __sdata_as_string( s1, SQL_CHAR, 
                             text_length_ptr, message_text ));
             }
@@ -716,13 +739,13 @@ SQLRETURN SQLGetDiagRec( SQLSMALLINT handle_type,
         {
             sprintf( statement -> msg, 
                 "\n\t\tEntry:\
-                \n\t\t\tStatement = %p\
-                \n\t\t\tRec Number = %d\
-                \n\t\t\tSQLState = %p\
-                \n\t\t\tNative = %p\
-                \n\t\t\tMessage Text = %p\
-                \n\t\t\tBuffer Length = %d\
-                \n\t\t\tText Len Ptr = %p",
+\n\t\t\tStatement = %p\
+\n\t\t\tRec Number = %d\
+\n\t\t\tSQLState = %p\
+\n\t\t\tNative = %p\
+\n\t\t\tMessage Text = %p\
+\n\t\t\tBuffer Length = %d\
+\n\t\t\tText Len Ptr = %p",
                     statement,
                     rec_number,
                     sqlstate,
@@ -752,12 +775,12 @@ SQLRETURN SQLGetDiagRec( SQLSMALLINT handle_type,
             {
                 sprintf( statement -> msg, 
                     "\n\t\tExit:[%s]\
-                    \n\t\t\tSQLState = %s\
-                    \n\t\t\tNative = %s\
-                    \n\t\t\tMessage Text = %s",
+\n\t\t\tSQLState = %s\
+\n\t\t\tNative = %s\
+\n\t\t\tMessage Text = %s",
                         __get_return_status( ret, s2 ),
                         sqlstate,
-                        __ptr_as_string( s0, native ),
+                        __iptr_as_string( s0, native ),
                         __sdata_as_string( s1, SQL_CHAR, 
                             text_length_ptr, message_text ));
             }
@@ -794,13 +817,13 @@ SQLRETURN SQLGetDiagRec( SQLSMALLINT handle_type,
         {
             sprintf( descriptor -> msg, 
                 "\n\t\tEntry:\
-                \n\t\t\tDescriptor = %p\
-                \n\t\t\tRec Number = %d\
-                \n\t\t\tSQLState = %p\
-                \n\t\t\tNative = %p\
-                \n\t\t\tMessage Text = %p\
-                \n\t\t\tBuffer Length = %d\
-                \n\t\t\tText Len Ptr = %p",
+\n\t\t\tDescriptor = %p\
+\n\t\t\tRec Number = %d\
+\n\t\t\tSQLState = %p\
+\n\t\t\tNative = %p\
+\n\t\t\tMessage Text = %p\
+\n\t\t\tBuffer Length = %d\
+\n\t\t\tText Len Ptr = %p",
                     descriptor,
                     rec_number,
                     sqlstate,
@@ -830,12 +853,12 @@ SQLRETURN SQLGetDiagRec( SQLSMALLINT handle_type,
             {
                 sprintf( descriptor -> msg, 
                     "\n\t\tExit:[%s]\
-                    \n\t\t\tSQLState = %s\
-                    \n\t\t\tNative = %s\
-                    \n\t\t\tMessage Text = %s",
+\n\t\t\tSQLState = %s\
+\n\t\t\tNative = %s\
+\n\t\t\tMessage Text = %s",
                         __get_return_status( ret, s2 ),
                         sqlstate,
-                        __ptr_as_string( s0, native ),
+                        __iptr_as_string( s0, native ),
                         __sdata_as_string( s1, SQL_CHAR, 
                             text_length_ptr, message_text ));
             }
