@@ -27,9 +27,21 @@
  *
  **********************************************************************
  *
- * $Id: SQLGetInfo.c,v 1.10 2006/08/31 12:44:52 lurcher Exp $
+ * $Id: SQLGetInfo.c,v 1.14 2009/02/18 17:59:08 lurcher Exp $
  *
  * $Log: SQLGetInfo.c,v $
+ * Revision 1.14  2009/02/18 17:59:08  lurcher
+ * Shift to using config.h, the compile lines were making it hard to spot warnings
+ *
+ * Revision 1.13  2009/02/17 09:47:44  lurcher
+ * Clear up a number of bugs
+ *
+ * Revision 1.12  2008/09/29 14:02:45  lurcher
+ * Fix missing dlfcn group option
+ *
+ * Revision 1.11  2007/01/02 10:29:18  lurcher
+ * Fix descriptor leak with unicode only driver
+ *
  * Revision 1.10  2006/08/31 12:44:52  lurcher
  * Check in for 2.2.12 release
  *
@@ -157,9 +169,10 @@
  *
  **********************************************************************/
 
+#include <config.h>
 #include "drivermanager.h"
 
-static char const rcsid[]= "$RCSfile: SQLGetInfo.c,v $ $Revision: 1.10 $";
+static char const rcsid[]= "$RCSfile: SQLGetInfo.c,v $ $Revision: 1.14 $";
 
 SQLRETURN SQLGetInfoA( SQLHDBC connection_handle,
            SQLUSMALLINT info_type,
@@ -174,7 +187,7 @@ SQLRETURN SQLGetInfoA( SQLHDBC connection_handle,
                 string_length );
 }
 
-SQLRETURN SQLGetInfo( SQLHDBC connection_handle,
+SQLRETURN __SQLGetInfo( SQLHDBC connection_handle,
            SQLUSMALLINT info_type,
            SQLPOINTER info_value,
            SQLSMALLINT buffer_length,
@@ -186,93 +199,6 @@ SQLRETURN SQLGetInfo( SQLHDBC connection_handle,
 	SQLUSMALLINT sval;
     char txt[ 30 ], *cptr;
     SQLPOINTER *ptr;
-    SQLCHAR s1[ 100 + LOG_MESSAGE_LEN ];
-
-    /*
-     * check connection
-     */
-
-    if ( !__validate_dbc( connection ))
-    {
-        dm_log_write( __FILE__, 
-                    __LINE__, 
-                    LOG_INFO, 
-                    LOG_INFO, 
-                    "Error: SQL_INVALID_HANDLE" );
-
-        return SQL_INVALID_HANDLE;
-    }
-
-    function_entry( connection );
-
-    if ( log_info.log_flag )
-    {
-        sprintf( connection -> msg, "\n\t\tEntry:\
-            \n\t\t\tConnection = %p\
-            \n\t\t\tInfo Type = %s (%d)\
-            \n\t\t\tInfo Value = %p\
-            \n\t\t\tBuffer Length = %d\
-            \n\t\t\tStrLen = %p",
-                connection,
-                __info_as_string( s1, info_type ),
-                info_type,
-                info_value, 
-                (int)buffer_length,
-                (void*)string_length );
-
-        dm_log_write( __FILE__, 
-                __LINE__, 
-                LOG_INFO, 
-                LOG_INFO, 
-                connection -> msg );
-    }
-
-    thread_protect( SQL_HANDLE_DBC, connection );
-
-    if ( info_type != SQL_ODBC_VER &&
-            connection -> state == STATE_C2 )
-    {
-        dm_log_write( __FILE__, 
-                __LINE__, 
-                LOG_INFO, 
-                LOG_INFO, 
-                "Error: 08003" );
-
-        __post_internal_error( &connection -> error,
-                ERROR_08003, NULL,
-                connection -> environment -> requested_version );
-
-        return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
-    }
-    else if ( connection -> state == STATE_C3 )
-    {
-        dm_log_write( __FILE__, 
-                __LINE__, 
-                LOG_INFO, 
-                LOG_INFO, 
-                "Error: 08003" );
-
-        __post_internal_error( &connection -> error,
-                ERROR_08003, NULL,
-                connection -> environment -> requested_version );
-
-        return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
-    }
-
-    if ( buffer_length < 0 )
-    {
-        dm_log_write( __FILE__, 
-                __LINE__, 
-                LOG_INFO, 
-                LOG_INFO, 
-                "Error: HY090" );
-
-        __post_internal_error( &connection -> error,
-                ERROR_HY090, NULL,
-                connection -> environment -> requested_version );
-
-        return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
-    }
 
     switch ( info_type )
     {
@@ -308,27 +234,30 @@ SQLRETURN SQLGetInfo( SQLHDBC connection_handle,
 
       case SQL_DRIVER_HDESC:
         {
-            DMHDESC hdesc = *((DMHDESC*) info_value); 
-
-            type = 2;
-
-            if ( __validate_desc( hdesc ))
+            if ( info_value ) 
             {
-                ptr = (SQLPOINTER) hdesc -> driver_desc;
-            }
-            else
-            {
-                dm_log_write( __FILE__, 
-                        __LINE__, 
-                        LOG_INFO, 
-                        LOG_INFO, 
-                        "Error: HY024" );
+                DMHDESC hdesc = *((DMHDESC*) info_value); 
 
-                __post_internal_error( &connection -> error,
-                        ERROR_HY024, NULL,
-                        connection -> environment -> requested_version );
+                type = 2;
 
-                return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
+                if ( __validate_desc( hdesc ))
+                {
+                    ptr = (SQLPOINTER) hdesc -> driver_desc;
+                }
+                else
+                {
+                    dm_log_write( __FILE__, 
+                            __LINE__, 
+                            LOG_INFO, 
+                            LOG_INFO, 
+                            "Error: HY024" );
+    
+                    __post_internal_error( &connection -> error,
+                            ERROR_HY024, NULL,
+                            connection -> environment -> requested_version );
+
+                    return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
+                }
             }
         }
         break;
@@ -340,27 +269,30 @@ SQLRETURN SQLGetInfo( SQLHDBC connection_handle,
 
       case SQL_DRIVER_HSTMT:
         {
-            DMHSTMT hstmt = *((DMHSTMT*) info_value); 
-
-            type = 2;
-
-            if ( __validate_stmt( hstmt ))
+            if ( info_value ) 
             {
-                ptr = (SQLPOINTER) hstmt -> driver_stmt;
-            }
-            else
-            {
-                dm_log_write( __FILE__, 
+                DMHSTMT hstmt = *((DMHSTMT*) info_value); 
+
+                type = 2;
+
+                if ( __validate_stmt( hstmt ))
+                {
+                    ptr = (SQLPOINTER) hstmt -> driver_stmt;
+                }
+                else
+                {
+                    dm_log_write( __FILE__, 
                         __LINE__, 
                         LOG_INFO, 
                         LOG_INFO, 
                         "Error: HY024" );
 
-                __post_internal_error( &connection -> error,
+                    __post_internal_error( &connection -> error,
                         ERROR_HY024, NULL,
                         connection -> environment -> requested_version );
 
-                return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
+                    return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
+                }
             }
         }
         break;
@@ -496,7 +428,7 @@ SQLRETURN SQLGetInfo( SQLHDBC connection_handle,
               case SQL_OUTER_JOINS:
                 if ( SQL_SUCCEEDED( ret ) && info_value && s1 )
                 {
-                    unicode_to_ansi_copy( info_value, s1, SQL_NTS, connection );
+                    unicode_to_ansi_copy( info_value, buffer_length, s1, SQL_NTS, connection );
                 }
 				if ( SQL_SUCCEEDED( ret ) && string_length && info_value ) 
 				{
@@ -534,19 +466,6 @@ SQLRETURN SQLGetInfo( SQLHDBC connection_handle,
                     string_length );
         }
 
-        if ( log_info.log_flag )
-        {
-            sprintf( connection -> msg, 
-                    "\n\t\tExit:[%s]",
-                        __get_return_status( ret, s1 ));
-
-            dm_log_write( __FILE__, 
-                    __LINE__, 
-                    LOG_INFO, 
-                    LOG_INFO, 
-                    connection -> msg );
-        }
-
         return function_return( SQL_HANDLE_DBC, connection, ret );
     }
 
@@ -586,6 +505,107 @@ SQLRETURN SQLGetInfo( SQLHDBC connection_handle,
             *string_length = sizeof( SQLUSMALLINT );
 	}
 
+    return function_return( SQL_HANDLE_DBC, connection, ret );
+}
+
+SQLRETURN SQLGetInfo( SQLHDBC connection_handle,
+           SQLUSMALLINT info_type,
+           SQLPOINTER info_value,
+           SQLSMALLINT buffer_length,
+           SQLSMALLINT *string_length )
+{
+    DMHDBC connection = (DMHDBC)connection_handle;
+    SQLRETURN ret = SQL_SUCCESS;
+    SQLCHAR s1[ 100 + LOG_MESSAGE_LEN ];
+
+    /*
+     * check connection
+     */
+
+    if ( !__validate_dbc( connection ))
+    {
+        dm_log_write( __FILE__, 
+                    __LINE__, 
+                    LOG_INFO, 
+                    LOG_INFO, 
+                    "Error: SQL_INVALID_HANDLE" );
+
+        return SQL_INVALID_HANDLE;
+    }
+
+    function_entry( connection );
+
+    if ( log_info.log_flag )
+    {
+        sprintf( connection -> msg, "\n\t\tEntry:\
+\n\t\t\tConnection = %p\
+\n\t\t\tInfo Type = %s (%d)\
+\n\t\t\tInfo Value = %p\
+\n\t\t\tBuffer Length = %d\
+\n\t\t\tStrLen = %p",
+                connection,
+                __info_as_string( s1, info_type ),
+                info_type,
+                info_value, 
+                (int)buffer_length,
+                (void*)string_length );
+
+        dm_log_write( __FILE__, 
+                __LINE__, 
+                LOG_INFO, 
+                LOG_INFO, 
+                connection -> msg );
+    }
+
+    thread_protect( SQL_HANDLE_DBC, connection );
+
+    if ( info_type != SQL_ODBC_VER &&
+            connection -> state == STATE_C2 )
+    {
+        dm_log_write( __FILE__, 
+                __LINE__, 
+                LOG_INFO, 
+                LOG_INFO, 
+                "Error: 08003" );
+
+        __post_internal_error( &connection -> error,
+                ERROR_08003, NULL,
+                connection -> environment -> requested_version );
+
+        return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
+    }
+    else if ( connection -> state == STATE_C3 )
+    {
+        dm_log_write( __FILE__, 
+                __LINE__, 
+                LOG_INFO, 
+                LOG_INFO, 
+                "Error: 08003" );
+
+        __post_internal_error( &connection -> error,
+                ERROR_08003, NULL,
+                connection -> environment -> requested_version );
+
+        return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
+    }
+
+    if ( buffer_length < 0 )
+    {
+        dm_log_write( __FILE__, 
+                __LINE__, 
+                LOG_INFO, 
+                LOG_INFO, 
+                "Error: HY090" );
+
+        __post_internal_error( &connection -> error,
+                ERROR_HY090, NULL,
+                connection -> environment -> requested_version );
+
+        return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
+    }
+
+	ret = __SQLGetInfo(connection_handle, info_type, info_value, buffer_length, string_length);
+
     if ( log_info.log_flag )
     {
         sprintf( connection -> msg, 
