@@ -27,9 +27,18 @@
  *
  **********************************************************************
  *
- * $Id: SQLGetInfoW.c,v 1.11 2005/10/06 08:50:58 lurcher Exp $
+ * $Id: SQLGetInfoW.c,v 1.14 2009/02/18 17:59:08 lurcher Exp $
  *
  * $Log: SQLGetInfoW.c,v $
+ * Revision 1.14  2009/02/18 17:59:08  lurcher
+ * Shift to using config.h, the compile lines were making it hard to spot warnings
+ *
+ * Revision 1.13  2008/08/29 08:01:39  lurcher
+ * Alter the way W functions are passed to the driver
+ *
+ * Revision 1.12  2007/02/28 15:37:48  lurcher
+ * deal with drivers that call internal W functions and end up in the driver manager. controlled by the --enable-handlemap configure arg
+ *
  * Revision 1.11  2005/10/06 08:50:58  lurcher
  * Fix problem with SQLDrivers not returning first entry
  *
@@ -97,6 +106,7 @@
  *
  **********************************************************************/
 
+#include <config.h>
 #include "drivermanager.h"
 
 static char const rcsid[]= "$RCSfile: SQLGetInfoW.c,v $";
@@ -127,6 +137,37 @@ SQLRETURN SQLGetInfoW( SQLHDBC connection_handle,
                     LOG_INFO, 
                     "Error: SQL_INVALID_HANDLE" );
 
+#ifdef WITH_HANDLE_REDIRECT
+		{
+			DMHDBC parent_connection;
+
+			parent_connection = find_parent_handle( connection, SQL_HANDLE_DBC );
+
+			if ( parent_connection ) {
+        		dm_log_write( __FILE__, 
+                	__LINE__, 
+                    	LOG_INFO, 
+                    	LOG_INFO, 
+                    	"Info: found parent handle" );
+
+				if ( CHECK_SQLGETINFOW( parent_connection ))
+				{
+        			dm_log_write( __FILE__, 
+                		__LINE__, 
+                   		 	LOG_INFO, 
+                   		 	LOG_INFO, 
+                   		 	"Info: calling redirected driver function" );
+
+					return SQLGETINFOW( parent_connection, 
+							connection_handle, 
+           					info_type,
+           					info_value,
+           					buffer_length,
+           					string_length );
+				}
+			}
+		}
+#endif
         return SQL_INVALID_HANDLE;
     }
 
@@ -135,11 +176,11 @@ SQLRETURN SQLGetInfoW( SQLHDBC connection_handle,
     if ( log_info.log_flag )
     {
         sprintf( connection -> msg, "\n\t\tEntry:\
-            \n\t\t\tConnection = %p\
-            \n\t\t\tInfo Type = %s\
-            \n\t\t\tInfo Value = %p\
-            \n\t\t\tBuffer Length = %d\
-            \n\t\t\tStrLen = %p",
+\n\t\t\tConnection = %p\
+\n\t\t\tInfo Type = %s\
+\n\t\t\tInfo Value = %p\
+\n\t\t\tBuffer Length = %d\
+\n\t\t\tStrLen = %p",
                 connection,
                 __info_as_string( s1, info_type ),
                 info_value, 
@@ -234,27 +275,30 @@ SQLRETURN SQLGetInfoW( SQLHDBC connection_handle,
 
       case SQL_DRIVER_HDESC:
         {
-            DMHDESC hdesc = *((DMHDESC*) info_value); 
-
-            type = 2;
-
-            if ( __validate_desc( hdesc ))
+            if ( info_value ) 
             {
-                ptr = (SQLPOINTER) hdesc -> driver_desc;
-            }
-            else
-            {
-                dm_log_write( __FILE__, 
-                        __LINE__, 
-                        LOG_INFO, 
-                        LOG_INFO, 
-                        "Error: HY024" );
+                DMHDESC hdesc = *((DMHDESC*) info_value); 
 
-                __post_internal_error( &connection -> error,
-                        ERROR_HY024, NULL,
-                        connection -> environment -> requested_version );
+                type = 2;
 
-                return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
+                if ( __validate_desc( hdesc ))
+                {
+                    ptr = (SQLPOINTER) hdesc -> driver_desc;
+                }
+                else
+                {
+                    dm_log_write( __FILE__, 
+                            __LINE__, 
+                            LOG_INFO, 
+                            LOG_INFO, 
+                            "Error: HY024" );
+
+                    __post_internal_error( &connection -> error,
+                            ERROR_HY024, NULL,
+                            connection -> environment -> requested_version );
+    
+                    return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
+                }
             }
         }
         break;
@@ -266,27 +310,30 @@ SQLRETURN SQLGetInfoW( SQLHDBC connection_handle,
 
       case SQL_DRIVER_HSTMT:
         {
-            DMHSTMT hstmt = *((DMHSTMT*) info_value); 
-
-            type = 2;
-
-            if ( __validate_stmt( hstmt ))
+            if ( info_value ) 
             {
-                ptr = (SQLPOINTER) hstmt -> driver_stmt;
-            }
-            else
-            {
-                dm_log_write( __FILE__, 
-                        __LINE__, 
-                        LOG_INFO, 
-                        LOG_INFO, 
-                        "Error: HY024" );
+                DMHSTMT hstmt = *((DMHSTMT*) info_value); 
 
-                __post_internal_error( &connection -> error,
-                        ERROR_HY024, NULL,
-                        connection -> environment -> requested_version );
+                type = 2;
 
-                return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
+                if ( __validate_stmt( hstmt ))
+                {
+                    ptr = (SQLPOINTER) hstmt -> driver_stmt;
+                }
+                else
+                {
+                    dm_log_write( __FILE__, 
+                            __LINE__, 
+                            LOG_INFO, 
+                            LOG_INFO, 
+                            "Error: HY024" );
+    
+                    __post_internal_error( &connection -> error,
+                            ERROR_HY024, NULL,
+                            connection -> environment -> requested_version );
+    
+                    return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
+                }
             }
         }
         break;
@@ -305,7 +352,8 @@ SQLRETURN SQLGetInfoW( SQLHDBC connection_handle,
         /*
          * pass all the others on
          */
-        if ( connection -> unicode_driver )
+        if ( connection -> unicode_driver ||
+			CHECK_SQLGETINFOW( connection ))
         {
             if ( !CHECK_SQLGETINFOW( connection ))
             {
