@@ -27,9 +27,15 @@
  *
  **********************************************************************
  *
- * $Id: SQLParamData.c,v 1.5 2005/11/21 17:25:43 lurcher Exp $
+ * $Id: SQLParamData.c,v 1.7 2009/02/18 17:59:08 lurcher Exp $
  *
  * $Log: SQLParamData.c,v $
+ * Revision 1.7  2009/02/18 17:59:08  lurcher
+ * Shift to using config.h, the compile lines were making it hard to spot warnings
+ *
+ * Revision 1.6  2007/05/25 16:42:32  lurcher
+ * Sync up
+ *
  * Revision 1.5  2005/11/21 17:25:43  lurcher
  * A few DM fixes for Oracle's ODBC driver
  *
@@ -119,9 +125,10 @@
  *
  **********************************************************************/
 
+#include <config.h>
 #include "drivermanager.h"
 
-static char const rcsid[]= "$RCSfile: SQLParamData.c,v $ $Revision: 1.5 $";
+static char const rcsid[]= "$RCSfile: SQLParamData.c,v $ $Revision: 1.7 $";
 
 SQLRETURN SQLParamData( SQLHSTMT statement_handle,
            SQLPOINTER *value )
@@ -150,8 +157,8 @@ SQLRETURN SQLParamData( SQLHSTMT statement_handle,
     if ( log_info.log_flag )
     {
         sprintf( statement -> msg, "\n\t\tEntry:\
-            \n\t\t\tStatement = %p\
-            \n\t\t\tValue = %p",
+\n\t\t\tStatement = %p\
+\n\t\t\tValue = %p",
                 statement,
                 value );
 
@@ -291,12 +298,18 @@ SQLRETURN SQLParamData( SQLHSTMT statement_handle,
         else
         {
             statement -> state = STATE_S6;
+            statement -> eod = 0;
         }
     }
     else if ( ret == SQL_NEED_DATA )
     {
         statement -> state = STATE_S9;
     }
+	else if ( ret == SQL_NO_DATA )
+	{
+		statement -> interupted_func = 0;
+		statement -> state = STATE_S4;
+	}
     else
     {
         if ( statement -> interupted_func == SQL_API_SQLEXECDIRECT )
@@ -327,6 +340,7 @@ SQLRETURN SQLParamData( SQLHSTMT statement_handle,
         else
         {
             statement -> state = STATE_S6;
+            statement -> eod = 0;
         }
     }
 
@@ -334,7 +348,7 @@ SQLRETURN SQLParamData( SQLHSTMT statement_handle,
     {
         sprintf( statement -> msg, 
                 "\n\t\tExit:[%s]\
-                \n\t\t\tValue = %p",
+\n\t\t\tValue = %p",
                     __get_return_status( ret, s1 ),
                     *value );
 
