@@ -27,9 +27,18 @@
  *
  **********************************************************************
  *
- * $Id: SQLSetConnectAttrW.c,v 1.11 2006/04/18 10:24:47 lurcher Exp $
+ * $Id: SQLSetConnectAttrW.c,v 1.14 2009/02/18 17:59:08 lurcher Exp $
  *
  * $Log: SQLSetConnectAttrW.c,v $
+ * Revision 1.14  2009/02/18 17:59:08  lurcher
+ * Shift to using config.h, the compile lines were making it hard to spot warnings
+ *
+ * Revision 1.13  2008/08/29 08:01:39  lurcher
+ * Alter the way W functions are passed to the driver
+ *
+ * Revision 1.12  2007/02/28 15:37:48  lurcher
+ * deal with drivers that call internal W functions and end up in the driver manager. controlled by the --enable-handlemap configure arg
+ *
  * Revision 1.11  2006/04/18 10:24:47  lurcher
  * Add a couple of changes from Mark Vanderwiel
  *
@@ -113,6 +122,7 @@
  *
  **********************************************************************/
 
+#include <config.h>
 #include "drivermanager.h"
 
 static char const rcsid[]= "$RCSfile: SQLSetConnectAttrW.c,v $";
@@ -133,6 +143,30 @@ SQLRETURN SQLSetConnectAttrW( SQLHDBC connection_handle,
 
     if ( attribute == SQL_ATTR_TRACE )
     {
+        if ((SQLLEN) value != SQL_OPT_TRACE_OFF && 
+            (SQLLEN) value != SQL_OPT_TRACE_ON ) 
+        {
+            if ( __validate_dbc( connection ))
+            {
+                thread_protect( SQL_HANDLE_DBC, connection );
+                dm_log_write( __FILE__, 
+                        __LINE__, 
+                        LOG_INFO, 
+                        LOG_INFO, 
+                        "Error: HY024" );
+        
+                __post_internal_error( &connection -> error,
+                    ERROR_HY024, NULL,
+                    connection -> environment -> requested_version );
+        
+                return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
+            }
+            else 
+            {
+                return SQL_INVALID_HANDLE;
+            }
+        }
+
         if ((SQLLEN) value == SQL_OPT_TRACE_OFF )
         {
             char force_string[ 30 ];
@@ -179,11 +213,58 @@ SQLRETURN SQLSetConnectAttrW( SQLHDBC connection_handle,
     {
         if ( value )
         {
-            if ( log_info.log_file_name )
+            if (((SQLWCHAR*)value)[ 0 ] == 0 ) 
             {
-                free( log_info.log_file_name );
+                if ( __validate_dbc( connection ))
+                {
+                    thread_protect( SQL_HANDLE_DBC, connection );
+                    dm_log_write( __FILE__, 
+                            __LINE__, 
+                            LOG_INFO, 
+                            LOG_INFO, 
+                            "Error: HY024" );
+            
+                    __post_internal_error( &connection -> error,
+                        ERROR_HY024, NULL,
+                        connection -> environment -> requested_version );
+            
+                    return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
+                }
+                else 
+                {
+                    return SQL_INVALID_HANDLE;
+                }
+            }
+            else 
+            {
+                if ( log_info.log_file_name )
+                {
+                    free( log_info.log_file_name );
+                }
+                log_info.log_file_name = unicode_to_ansi_alloc((SQLWCHAR *) value, SQL_NTS, connection );
+            }
+        }
+        else 
+        {
+            if ( __validate_dbc( connection ))
+            {
+                thread_protect( SQL_HANDLE_DBC, connection );
+                dm_log_write( __FILE__, 
+                        __LINE__, 
+                        LOG_INFO, 
+                        LOG_INFO, 
+                        "Error: HY009" );
+        
+                __post_internal_error( &connection -> error,
+                    ERROR_HY009, NULL,
+                    connection -> environment -> requested_version );
+        
+                return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
+            }
+            else 
+            {
+                return SQL_INVALID_HANDLE;
             }
-            log_info.log_file_name = unicode_to_ansi_alloc( value, SQL_NTS, connection );
         }
         return SQL_SUCCESS;
     }
@@ -200,6 +281,36 @@ SQLRETURN SQLSetConnectAttrW( SQLHDBC connection_handle,
                     LOG_INFO, 
                     "Error: SQL_INVALID_HANDLE" );
 
+#ifdef WITH_HANDLE_REDIRECT
+		{
+			DMHDBC parent_connection;
+
+			parent_connection = find_parent_handle( connection, SQL_HANDLE_DBC );
+
+			if ( parent_connection ) {
+        		dm_log_write( __FILE__, 
+                	__LINE__, 
+                    	LOG_INFO, 
+                    	LOG_INFO, 
+                    	"Info: found parent handle" );
+
+				if ( CHECK_SQLSETCONNECTATTRW( parent_connection ))
+				{
+        			dm_log_write( __FILE__, 
+                		__LINE__, 
+                   		 	LOG_INFO, 
+                   		 	LOG_INFO, 
+                   		 	"Info: calling redirected driver function" );
+
+					return SQLSETCONNECTATTRW( parent_connection, 
+							connection_handle, 
+           					attribute,
+           					value,
+           					string_length );
+				}
+			}
+		}
+#endif
         return SQL_INVALID_HANDLE;
     }
 
@@ -208,14 +319,14 @@ SQLRETURN SQLSetConnectAttrW( SQLHDBC connection_handle,
     if ( log_info.log_flag )
     {
         sprintf( connection -> msg, "\n\t\tEntry:\
-            \n\t\t\tConnection = %p\
-            \n\t\t\tAttribute = %s\
-            \n\t\t\tValue = %p\
-            \n\t\t\tStrLen = %d",
+\n\t\t\tConnection = %p\
+\n\t\t\tAttribute = %s\
+\n\t\t\tValue = %p\
+\n\t\t\tStrLen = %d",
                 connection,
                 __con_attr_as_string( s1, attribute ),
                 value, 
-                string_length );
+                (int)string_length );
 
         dm_log_write( __FILE__, 
                 __LINE__, 
@@ -292,6 +403,108 @@ SQLRETURN SQLSetConnectAttrW( SQLHDBC connection_handle,
         }
     }
 
+    /*
+     * is it a legitimate value
+     */
+    ret = dm_check_connection_attrs( connection, attribute, value );
+
+    if ( ret != SQL_SUCCESS ) 
+    {
+        dm_log_write( __FILE__, 
+                    __LINE__, 
+                    LOG_INFO, 
+                    LOG_INFO, 
+                    "Error: HY024" );
+
+        __post_internal_error( &connection -> error,
+                ERROR_HY024, NULL,
+                connection -> environment -> requested_version );
+
+        return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
+    }
+
+    /*
+     * is it a connection attribute or statement, check state of any active connections
+     */
+
+    switch( attribute ) 
+    {
+      	case SQL_ATTR_CONCURRENCY:
+	  	case SQL_BIND_TYPE:
+      	case SQL_ATTR_CURSOR_SCROLLABLE:
+      	case SQL_ATTR_CURSOR_SENSITIVITY:
+      	case SQL_ATTR_CURSOR_TYPE:
+      	case SQL_ATTR_MAX_LENGTH:
+      	case SQL_MAX_ROWS:
+      	case SQL_ATTR_KEYSET_SIZE:
+      	case SQL_ROWSET_SIZE: 
+      	case SQL_ATTR_NOSCAN:
+      	case SQL_ATTR_QUERY_TIMEOUT:
+      	case SQL_ATTR_RETRIEVE_DATA:
+      	case SQL_ATTR_SIMULATE_CURSOR:
+      	case SQL_ATTR_USE_BOOKMARKS:
+            if( __check_stmt_from_dbc( connection, STATE_S8 ) ||
+                __check_stmt_from_dbc( connection, STATE_S9 ) ||
+                __check_stmt_from_dbc( connection, STATE_S10 ) ||
+                __check_stmt_from_dbc( connection, STATE_S11 ) ||
+                __check_stmt_from_dbc( connection, STATE_S12 )) {
+
+                dm_log_write( __FILE__, 
+                        __LINE__, 
+                        LOG_INFO, 
+                        LOG_INFO, 
+                        "Error: 24000" );
+
+                __post_internal_error( &connection -> error,
+                        ERROR_24000, NULL,
+                        connection -> environment -> requested_version );
+
+                return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
+            }
+            break;
+
+        default:
+            if ( attribute == SQL_ATTR_CURRENT_CATALOG ) 
+            {
+                if( __check_stmt_from_dbc( connection, STATE_S5 ) ||
+                    __check_stmt_from_dbc( connection, STATE_S6 ) ||
+                    __check_stmt_from_dbc( connection, STATE_S7 )) {
+
+                    dm_log_write( __FILE__, 
+                            __LINE__, 
+                            LOG_INFO, 
+                            LOG_INFO, 
+                            "Error: 24000" );
+
+                    __post_internal_error( &connection -> error,
+                            ERROR_24000, NULL,
+                            connection -> environment -> requested_version );
+
+                    return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
+                }
+            }
+
+            if( __check_stmt_from_dbc( connection, STATE_S8 ) ||
+                __check_stmt_from_dbc( connection, STATE_S9 ) ||
+                __check_stmt_from_dbc( connection, STATE_S10 ) ||
+                __check_stmt_from_dbc( connection, STATE_S11 ) ||
+                __check_stmt_from_dbc( connection, STATE_S12 )) {
+
+                dm_log_write( __FILE__, 
+                        __LINE__, 
+                        LOG_INFO, 
+                        LOG_INFO, 
+                        "Error: HY010" );
+
+                __post_internal_error( &connection -> error,
+                        ERROR_HY010, NULL,
+                        connection -> environment -> requested_version );
+
+                return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
+            }
+            break;
+    }
+
     /*
      * is it something overridden
      */
@@ -409,7 +622,8 @@ SQLRETURN SQLSetConnectAttrW( SQLHDBC connection_handle,
     }
     else
     {
-        if ( connection -> unicode_driver )
+        if ( connection -> unicode_driver ||
+			CHECK_SQLSETCONNECTATTRW( connection ))
         {
             if ( !CHECK_SQLSETCONNECTATTRW( connection ))
             {
