@@ -27,9 +27,15 @@
  *
  **********************************************************************
  *
- * $Id: SQLSetScrollOptions.c,v 1.7 2005/11/23 08:29:16 lurcher Exp $
+ * $Id: SQLSetScrollOptions.c,v 1.9 2009/02/18 17:59:08 lurcher Exp $
  *
  * $Log: SQLSetScrollOptions.c,v $
+ * Revision 1.9  2009/02/18 17:59:08  lurcher
+ * Shift to using config.h, the compile lines were making it hard to spot warnings
+ *
+ * Revision 1.8  2007/01/02 10:27:50  lurcher
+ * Fix descriptor leak with unicode only driver
+ *
  * Revision 1.7  2005/11/23 08:29:16  lurcher
  * Add cleanup in postgres driver
  *
@@ -130,9 +136,10 @@
  *
  **********************************************************************/
 
+#include <config.h>
 #include "drivermanager.h"
 
-static char const rcsid[]= "$RCSfile: SQLSetScrollOptions.c,v $ $Revision: 1.7 $";
+static char const rcsid[]= "$RCSfile: SQLSetScrollOptions.c,v $ $Revision: 1.9 $";
 
 SQLRETURN SQLSetScrollOptions(
     SQLHSTMT           statement_handle,
@@ -164,10 +171,10 @@ SQLRETURN SQLSetScrollOptions(
     if ( log_info.log_flag )
     {
         sprintf( statement -> msg, "\n\t\tEntry:\
-            \n\t\t\tStatement = %p\
-            \n\t\t\tConcurrency = %d\
-            \n\t\t\tKeyset = %d\
-            \n\t\t\tRowset = %d",
+\n\t\t\tStatement = %p\
+\n\t\t\tConcurrency = %d\
+\n\t\t\tKeyset = %d\
+\n\t\t\tRowset = %d",
                 statement,
                 f_concurrency,
                 (int)crow_keyset,
@@ -246,7 +253,8 @@ SQLRETURN SQLSetScrollOptions(
                 crow_rowset );
     }
     else if ( statement -> connection -> driver_act_ver == SQL_OV_ODBC3 &&
-            CHECK_SQLGETINFO( statement -> connection ) &&
+            (CHECK_SQLGETINFO( statement -> connection ) || 
+			CHECK_SQLGETINFOW( statement -> connection )) &&
             (CHECK_SQLSETSTMTATTR( statement -> connection ) ||
             CHECK_SQLSETSTMTATTRW( statement -> connection )))
     {
@@ -292,8 +300,7 @@ SQLRETURN SQLSetScrollOptions(
             break;
         }
 
-        ret = SQLGETINFO( statement -> connection,
-                statement -> connection -> driver_dbc,
+        ret = __SQLGetInfo( statement -> connection,
                 info_type,
                 &ivp,
                 sizeof( ivp ),
