@@ -27,9 +27,21 @@
  *
  **********************************************************************
  *
- * $Id: SQLSetStmtAttrW.c,v 1.6 2006/04/18 10:24:47 lurcher Exp $
+ * $Id: SQLSetStmtAttrW.c,v 1.10 2009/02/18 17:59:08 lurcher Exp $
  *
  * $Log: SQLSetStmtAttrW.c,v $
+ * Revision 1.10  2009/02/18 17:59:08  lurcher
+ * Shift to using config.h, the compile lines were making it hard to spot warnings
+ *
+ * Revision 1.9  2009/02/04 09:30:02  lurcher
+ * Fix some SQLINTEGER/SQLLEN conflicts
+ *
+ * Revision 1.8  2008/08/29 08:01:39  lurcher
+ * Alter the way W functions are passed to the driver
+ *
+ * Revision 1.7  2007/02/28 15:37:48  lurcher
+ * deal with drivers that call internal W functions and end up in the driver manager. controlled by the --enable-handlemap configure arg
+ *
  * Revision 1.6  2006/04/18 10:24:47  lurcher
  * Add a couple of changes from Mark Vanderwiel
  *
@@ -76,6 +88,7 @@
  *
  **********************************************************************/
 
+#include <config.h>
 #include "drivermanager.h"
 
 static char const rcsid[]= "$RCSfile: SQLSetStmtAttrW.c,v $";
@@ -102,6 +115,36 @@ SQLRETURN SQLSetStmtAttrW( SQLHSTMT statement_handle,
                     LOG_INFO, 
                     "Error: SQL_INVALID_HANDLE" );
 
+#ifdef WITH_HANDLE_REDIRECT
+		{
+			DMHSTMT parent_statement;
+
+			parent_statement = find_parent_handle( statement, SQL_HANDLE_STMT );
+
+			if ( parent_statement ) {
+        		dm_log_write( __FILE__, 
+                	__LINE__, 
+                    	LOG_INFO, 
+                    	LOG_INFO, 
+                    	"Info: found parent handle" );
+
+				if ( CHECK_SQLSETSTMTATTRW( parent_statement -> connection ))
+				{
+        			dm_log_write( __FILE__, 
+                		__LINE__, 
+                   		 	LOG_INFO, 
+                   		 	LOG_INFO, 
+                   		 	"Info: calling redirected driver function" );
+
+                	return  SQLSETSTMTATTRW( parent_statement -> connection,
+							statement_handle,
+							attribute,
+							value,
+							string_length );
+				}
+			}
+		}
+#endif
         return SQL_INVALID_HANDLE;
     }
 
@@ -110,10 +153,10 @@ SQLRETURN SQLSetStmtAttrW( SQLHSTMT statement_handle,
     if ( log_info.log_flag )
     {
         sprintf( statement -> msg, "\n\t\tEntry:\
-            \n\t\t\tStatement = %p\
-            \n\t\t\tAttribute = %s\
-            \n\t\t\tValue = %p\
-            \n\t\t\tStrLen = %d",
+\n\t\t\tStatement = %p\
+\n\t\t\tAttribute = %s\
+\n\t\t\tValue = %p\
+\n\t\t\tStrLen = %d",
                 statement,
                 __stmt_attr_as_string( s1, attribute ),
                 value, 
@@ -229,7 +272,8 @@ SQLRETURN SQLSetStmtAttrW( SQLHSTMT statement_handle,
         }
     }
 
-    if ( statement -> connection -> unicode_driver )
+    if ( statement -> connection -> unicode_driver ||
+		    CHECK_SQLSETSTMTATTRW( statement -> connection ))
 	{
     	if ( !CHECK_SQLSETSTMTATTRW( statement -> connection ))
     	{
@@ -316,6 +360,7 @@ SQLRETURN SQLSetStmtAttrW( SQLHSTMT statement_handle,
          */
         value = ( SQLPOINTER ) desc -> driver_desc;
         statement -> ard = desc;
+        desc -> associated_with = statement;
     }
 
     if ( attribute == SQL_ATTR_APP_PARAM_DESC )
@@ -376,6 +421,7 @@ SQLRETURN SQLSetStmtAttrW( SQLHSTMT statement_handle,
          */
         value = ( SQLPOINTER ) desc -> driver_desc;
         statement -> apd = desc;
+        desc -> associated_with = statement;
     }
 
     /*
@@ -387,6 +433,42 @@ SQLRETURN SQLSetStmtAttrW( SQLHSTMT statement_handle,
         statement -> metadata_id = (long int) value;
     }
 
+    if ( attribute == SQL_ATTR_IMP_ROW_DESC || 
+        attribute == SQL_ATTR_IMP_PARAM_DESC )
+    {
+        dm_log_write( __FILE__, 
+                    __LINE__, 
+                    LOG_INFO, 
+                    LOG_INFO, 
+                    "Error: HY017" );
+
+        __post_internal_error( &statement -> error,
+                ERROR_HY017, NULL,
+                statement -> connection -> environment -> requested_version );
+
+        return function_return( SQL_HANDLE_STMT, statement, SQL_ERROR );
+    }
+
+    /*
+     * is it a legitimate value
+     */
+    ret = dm_check_statement_attrs( statement, attribute, value );
+
+    if ( ret != SQL_SUCCESS ) 
+    {
+        dm_log_write( __FILE__, 
+                    __LINE__, 
+                    LOG_INFO, 
+                    LOG_INFO, 
+                    "Error: HY011" );
+
+        __post_internal_error( &statement -> error,
+                ERROR_HY024, NULL,
+                statement -> connection -> environment -> requested_version );
+
+        return function_return( SQL_HANDLE_STMT, statement, SQL_ERROR );
+    }
+
     /*
      * is it something overridden
      */
@@ -402,7 +484,7 @@ SQLRETURN SQLSetStmtAttrW( SQLHSTMT statement_handle,
             CHECK_SQLEXTENDEDFETCH( statement -> connection ) &&
             !CHECK_SQLFETCHSCROLL( statement -> connection ))
     {
-        statement -> fetch_bm_ptr = (SQLUINTEGER*) value;
+        statement -> fetch_bm_ptr = (SQLULEN*) value;
         ret = SQL_SUCCESS;
     }
     else if ( attribute == SQL_ATTR_ROW_STATUS_PTR &&
@@ -414,7 +496,7 @@ SQLRETURN SQLSetStmtAttrW( SQLHSTMT statement_handle,
     else if ( attribute == SQL_ATTR_ROWS_FETCHED_PTR &&
 	      statement -> connection -> driver_act_ver == SQL_OV_ODBC2 )
     {
-        statement -> row_ct_ptr = (SQLUINTEGER*) value;
+        statement -> row_ct_ptr = (SQLULEN*) value;
         ret = SQL_SUCCESS;
     }
     else if ( attribute == SQL_ATTR_ROW_ARRAY_SIZE &&
