@@ -27,9 +27,15 @@
  *
  **********************************************************************
  *
- * $Id: __connection.c,v 1.4 2004/09/08 16:38:54 lurcher Exp $
+ * $Id: __connection.c,v 1.6 2009/02/18 17:59:08 lurcher Exp $
  *
  * $Log: __connection.c,v $
+ * Revision 1.6  2009/02/18 17:59:08  lurcher
+ * Shift to using config.h, the compile lines were making it hard to spot warnings
+ *
+ * Revision 1.5  2008/09/29 14:02:45  lurcher
+ * Fix missing dlfcn group option
+ *
  * Revision 1.4  2004/09/08 16:38:54  lurcher
  *
  * Get ready for a 2.2.10 release
@@ -89,6 +95,7 @@
  *
  **********************************************************************/
 
+#include <config.h>
 #include "drivermanager.h"
 
 /*
@@ -97,7 +104,7 @@
  * variable
  */
 
-static char const rcsid[]= "$RCSfile: __connection.c,v $ $Revision: 1.4 $";
+static char const rcsid[]= "$RCSfile: __connection.c,v $ $Revision: 1.6 $";
 
 /*
  * search for the library (.so) that the DSN points to
@@ -143,6 +150,10 @@ char *__find_lib_name( char *dsn, char *lib_name, char *driver_name )
 	{
         strcpy( driver, driver_lib );
 
+		/*
+		 * allow the use of "User odbcinst files
+		 */
+
 #ifdef PLATFORM64
 		SQLGetPrivateProfileString( driver, "Driver64", "",
 				driver_lib, sizeof( driver_lib ), "ODBCINST.INI" );
@@ -157,10 +168,11 @@ char *__find_lib_name( char *dsn, char *lib_name, char *driver_name )
 				driver_lib, sizeof( driver_lib ), "ODBCINST.INI" );
 #endif
 
-        strcpy( driver_name, driver );
+                strcpy( driver_name, driver );
 
-		if ( driver_lib[ 0 ] == 0 )
-			return NULL;
+		if ( driver_lib[ 0 ] == 0 ) {
+		    return NULL;
+		}
 	}
 
 	strcpy( lib_name, driver_lib );
