@@ -27,9 +27,21 @@
  *
  **********************************************************************
  *
- * $Id: __handles.c,v 1.9 2006/05/31 17:35:34 lurcher Exp $
+ * $Id: __handles.c,v 1.13 2009/05/15 15:23:56 lurcher Exp $
  *
  * $Log: __handles.c,v $
+ * Revision 1.13  2009/05/15 15:23:56  lurcher
+ * Fix pooled connection thread problems
+ *
+ * Revision 1.12  2009/02/18 17:59:08  lurcher
+ * Shift to using config.h, the compile lines were making it hard to spot warnings
+ *
+ * Revision 1.11  2009/02/17 09:47:44  lurcher
+ * Clear up a number of bugs
+ *
+ * Revision 1.10  2007/02/28 15:37:49  lurcher
+ * deal with drivers that call internal W functions and end up in the driver manager. controlled by the --enable-handlemap configure arg
+ *
  * Revision 1.9  2006/05/31 17:35:34  lurcher
  * Add unicode ODBCINST entry points
  *
@@ -206,6 +218,7 @@
  *
  **********************************************************************/
 
+#include <config.h>
 #include <ctype.h>
 #include "drivermanager.h"
 #if defined ( COLLECT_STATS ) && defined( HAVE_SYS_SEM_H )
@@ -213,7 +226,7 @@
 #include <uodbc_stats.h>
 #endif
 
-static char const rcsid[]= "$RCSfile: __handles.c,v $ $Revision: 1.9 $";
+static char const rcsid[]= "$RCSfile: __handles.c,v $ $Revision: 1.13 $";
 
 /*
  * these are used to enable us to check if a handle is
@@ -380,7 +393,6 @@ void mutex_lib_exit( void )
 DMHENV __alloc_env( void )
 {
     DMHENV environment = NULL;
-    char s0[ 20 ];
 
     mutex_entry( &mutex_lists );
 
@@ -620,40 +632,35 @@ DMHDBC __alloc_dbc( void )
 void dbc_change_thread_support( DMHDBC connection, int level )
 {
 #if defined ( HAVE_LIBPTHREAD ) || defined( HAVE_LIBTHREAD ) || defined( HAVE_LIBPTH )
+    int old_level;
 
     if ( connection -> protection_level == level )
-        return;
+	return;
 
-    mutex_entry( &mutex_lists );
-
-    /*
-     * switch level
-     * If the previous level was at less than connection level,
-     * we need to create a lock at the environment level then release
-     * the connection lock.
-     * 
-     * If we are moving from a greater than env lock, the current lock
-     * on the connection will be ok
-     */ 
+    old_level =  connection -> protection_level;
+    connection -> protection_level = level;
 
     if ( level == TS_LEVEL3 )
     {
+	/*
+         * if we are moving from level 3 we may have to release the existing
+         * connection lock, and create the env lock
+         */
+	if(old_level != TS_LEVEL0)
+            mutex_exit( &connection -> mutex );
         mutex_entry( &mutex_env );
-        mutex_exit( &connection -> mutex );
     }
-    else if ( connection -> protection_level == TS_LEVEL3 )
+    else if ( old_level == TS_LEVEL3 )
     {
-        /*
-         * if we are moving from level 3 we have to create the new
+         /*
+         * if we are moving from level 3 we may have to create the new
          * connection lock, and remove the env lock
          */
-        mutex_entry( &connection -> mutex );
+	if(level != TS_LEVEL0)
+	    mutex_entry( &connection -> mutex );
         mutex_exit( &mutex_env );
     }
-    connection -> protection_level = level;
 
-    mutex_exit( &mutex_lists );
-                        
 #endif
 }
 
@@ -852,6 +859,106 @@ int __clean_stmt_from_dbc( DMHDBC connection )
     return ret;
 }
 
+/*
+ * check if any statements on this connection are in a given state
+ */
+
+int __check_stmt_from_dbc( DMHDBC connection, int state )
+{
+    DMHSTMT ptr, last;
+    int ret = 0;
+    int found = 0;
+
+    mutex_entry( &mutex_lists );
+    last = NULL;
+    ptr = statement_root;
+
+    while( ptr )
+    {
+        if ( ptr -> connection == connection )
+        {
+            if ( ptr -> state == state ) {
+                found = 1;
+                break;
+            }
+        }
+
+        ptr = ptr -> next_class_list;
+    }
+
+    mutex_exit( &mutex_lists );
+
+    return found;
+}
+
+int __check_stmt_from_desc( DMHDESC desc, int state )
+{
+    DMHDBC connection;
+    DMHSTMT ptr, last;
+    int ret = 0;
+    int found = 0;
+
+    mutex_entry( &mutex_lists );
+    last = NULL;
+    ptr = statement_root;
+    connection = desc -> connection;
+
+    while( ptr )
+    {
+        if ( ptr -> connection == connection )
+        {
+            if ( ptr -> ipd == desc || ptr -> ird == desc || ptr -> apd == desc || ptr -> ard == desc ) {
+                if ( ptr -> state == state ) {
+                    found = 1;
+                    break;
+                }
+            }
+        }
+
+        ptr = ptr -> next_class_list;
+    }
+
+    mutex_exit( &mutex_lists );
+
+    return found;
+}
+
+int __check_stmt_from_desc_ird( DMHDESC desc, int state )
+{
+    DMHDBC connection;
+    DMHSTMT ptr, last;
+    int ret = 0;
+    int found = 0;
+
+    mutex_entry( &mutex_lists );
+    last = NULL;
+    ptr = statement_root;
+    connection = desc -> connection;
+
+    while( ptr )
+    {
+        if ( ptr -> connection == connection )
+        {
+            if ( ptr -> ird == desc ) {
+                if ( ptr -> state == state ) {
+                    found = 1;
+                    break;
+                }
+            }
+        }
+
+        ptr = ptr -> next_class_list;
+    }
+
+    mutex_exit( &mutex_lists );
+
+    return found;
+}
+
+/*
+ * check any statements that are associated with a descriptor
+ */
+
 /*
  * check that a statement is real
  */
@@ -1140,7 +1247,6 @@ void __release_desc( DMHDESC descriptor )
 
 void thread_protect( int type, void *handle )
 {
-    DMHENV environment;
     DMHDBC connection;
     DMHSTMT statement;
     DMHDESC descriptor;
@@ -1200,7 +1306,6 @@ void thread_protect( int type, void *handle )
 
 void thread_release( int type, void *handle )
 {
-    DMHENV environment;
     DMHDBC connection;
     DMHSTMT statement;
     DMHDESC descriptor;
@@ -1259,3 +1364,69 @@ void thread_release( int type, void *handle )
 }
 
 #endif
+
+#ifdef WITH_HANDLE_REDIRECT
+
+/*
+ * try and find a handle that has the suplied handle as the driver handle
+ * there will be threading issues with this, so be carefull. 
+ * However it will normally only get used with "broken" drivers.
+ */
+
+
+void *find_parent_handle( DRV_SQLHANDLE drv_hand, int type )
+{
+	void *found_handle = NULL;
+
+    mutex_entry( &mutex_lists );
+
+	switch( type ) {
+		case SQL_HANDLE_DBC:
+			{
+				DMHDBC hand = connection_root;
+				while( hand ) {
+					if ( hand -> driver_dbc == drv_hand ) {
+						found_handle = hand;
+						break;
+					}
+					hand = hand -> next_class_list;
+				}
+			}
+			break;
+
+		case SQL_HANDLE_STMT:
+			{
+				DMHSTMT hand = statement_root;
+				while( hand ) {
+					if ( hand -> driver_stmt == drv_hand ) {
+						found_handle = hand;
+						break;
+					}
+					hand = hand -> next_class_list;
+				}
+			}
+			break;
+
+		case SQL_HANDLE_DESC:
+			{
+				DMHDESC hand = descriptor_root;
+				while( hand ) {
+					if ( hand -> driver_desc == drv_hand ) {
+						found_handle = hand;
+						break;
+					}
+					hand = hand -> next_class_list;
+				}
+			}
+			break;
+
+		default:
+			break;
+	}
+
+    mutex_exit( &mutex_lists );
+
+	return found_handle;
+}
+
+#endif
