@@ -1,7 +1,7 @@
 #ifndef _DRIVERMANAGER_H
 #define _DRIVERMANAGER_H
 
-#define ODBCVER 0x0351
+#define ODBCVER 0x0380
 
 #include <sys/types.h>
 #include <pwd.h>
@@ -252,7 +252,7 @@ void *__attr_override_wide( void *handle, int type, int attribute, void * value,
 struct env_lib_struct
 {
     char            *lib_name;
-    void            *env_handle;
+    DRV_SQLHANDLE   env_handle;
     int             count;
     struct env_lib_struct   *next;
 };
@@ -399,6 +399,7 @@ typedef struct descriptor
     DRV_SQLHDESC    driver_desc;        /* driver descriptor */
     DMHDBC          connection;         /* DM connection that owns this */
     int             implicit;           /* created by a AllocStmt */
+    void            *associated_with;   /* statement that this is a descriptor of */
 #ifdef HAVE_LIBPTH
     pth_mutex_t     mutex;              /* protect the object */
 #elif HAVE_LIBPTHREAD
@@ -433,10 +434,10 @@ typedef struct statement
     DMHDESC         implicit_apd;
     DMHDESC         implicit_ird;
     DMHDESC         implicit_ard;
-    SQLUINTEGER     *fetch_bm_ptr;      /* Saved for ODBC3 to ODBC2 mapping */ 
-    SQLUINTEGER     *row_ct_ptr;        /* row count ptr */
+    SQLULEN		    *fetch_bm_ptr;      /* Saved for ODBC3 to ODBC2 mapping */ 
+    SQLULEN     	*row_ct_ptr;        /* row count ptr */
     SQLUSMALLINT    *row_st_arr;        /* row status array */
-    SQLUINTEGER     row_array_size;
+    SQLULEN     	row_array_size;
 	SQLPOINTER      valueptr;           /* Default buffer for SQLParamData() */
 
 #ifdef HAVE_LIBPTH
@@ -445,7 +446,10 @@ typedef struct statement
     pthread_mutex_t mutex;              /* protect the object */
 #elif HAVE_LIBTHREAD
     mutex_t mutex;              		/* protect the object */
+
 #endif
+
+    int             eod;                /* when in S6 has EOD been returned */
 } *DMHSTMT;
 
 #if defined ( HAVE_LIBPTHREAD ) || defined ( HAVE_LIBTHREAD ) || defined ( HAVE_LIBPTH )
@@ -510,6 +514,12 @@ SQLRETURN __SQLAllocHandle( SQLSMALLINT handle_type,
 SQLRETURN __SQLFreeHandle( SQLSMALLINT handle_type,
            SQLHANDLE handle );
 
+SQLRETURN __SQLGetInfo( SQLHDBC connection_handle,
+		 	SQLUSMALLINT info_type,
+			SQLPOINTER info_value,
+			SQLSMALLINT buffer_length,
+			SQLSMALLINT *string_length );
+
 int __connect_part_one( DMHDBC connection, char *driver_lib, char *driver_name, int *warnings );
 void __disconnect_part_one( DMHDBC connection );
 int __connect_part_two( DMHDBC connection );
@@ -566,6 +576,7 @@ typedef enum error_id
     ERROR_HY001,
     ERROR_HY003,
     ERROR_HY004,
+    ERROR_HY007,
     ERROR_HY009,
     ERROR_HY010,
     ERROR_HY011,
@@ -595,7 +606,9 @@ typedef enum error_id
     ERROR_IM012,
     ERROR_SL004,
     ERROR_SL009,
-    ERROR_SL010
+    ERROR_SL010,
+    ERROR_SL008,
+    ERROR_HY000
 } error_id;
 
 #define IGNORE_THREAD       (-1)
@@ -627,7 +640,7 @@ void setup_error_head( EHEAD *error_header, void *handle, int handle_type );
 void clear_error_head( EHEAD *error_header );
 SQLWCHAR *ansi_to_unicode_copy( SQLWCHAR * dest, char *src, SQLINTEGER buffer_len, DMHDBC connection );
 SQLWCHAR *ansi_to_unicode_alloc( SQLCHAR *str, SQLINTEGER len, DMHDBC connection );
-char *unicode_to_ansi_copy( char* dest, SQLWCHAR *src, SQLINTEGER len, DMHDBC connection );
+char *unicode_to_ansi_copy( char* dest, int dest_len, SQLWCHAR *src, SQLINTEGER len, DMHDBC connection );
 char *unicode_to_ansi_alloc( SQLWCHAR *str, SQLINTEGER len, DMHDBC connection );
 int unicode_setup( DMHDBC connection );
 void unicode_shutdown( DMHDBC connection );
@@ -647,7 +660,8 @@ SQLWCHAR *wide_strdup( SQLWCHAR *str1 );
 int wide_strlen( SQLWCHAR *str1 );
 int wide_ansi_strncmp( SQLWCHAR *str1, char *str2, int len );
 char * __get_pid( SQLCHAR *str );
-char * __ptr_as_string( SQLCHAR *s, SQLINTEGER *ptr );
+char * __iptr_as_string( SQLCHAR *s, SQLINTEGER *ptr );
+char * __ptr_as_string( SQLCHAR *s, SQLLEN *ptr );
 char * __sptr_as_string( SQLCHAR *s, SQLSMALLINT *ptr );
 char * __info_as_string( SQLCHAR *s, SQLINTEGER typ );
 void __clear_internal_error( struct error *error_handle );
@@ -655,6 +669,8 @@ char * __data_as_string( SQLCHAR *s, SQLINTEGER type,
         SQLLEN *ptr, SQLPOINTER buf );
 char * __sdata_as_string( SQLCHAR *s, SQLINTEGER type, 
         SQLSMALLINT *ptr, SQLPOINTER buf );
+char * __idata_as_string( SQLCHAR *s, SQLINTEGER type, 
+        SQLINTEGER *ptr, SQLPOINTER buf );
 char * __col_attr_as_string( SQLCHAR *s, SQLINTEGER type );
 char * __fid_as_string( SQLCHAR *s, SQLINTEGER fid );
 char * __con_attr_as_string( SQLCHAR *s, SQLINTEGER type );
@@ -664,8 +680,13 @@ char * __desc_attr_as_string( SQLCHAR *s, SQLINTEGER type );
 char * __diag_attr_as_string( SQLCHAR *s, SQLINTEGER type );
 char * __type_as_string( SQLCHAR *s, SQLSMALLINT type );
 DMHDBC __get_connection( EHEAD * head );
-SQLHANDLE __get_driver_handle( EHEAD * head );
+DRV_SQLHANDLE __get_driver_handle( EHEAD * head );
 int __get_version( EHEAD * head );
+int dm_check_connection_attrs( DMHDBC connection, SQLINTEGER attribute, SQLPOINTER value );
+int dm_check_statement_attrs( DMHSTMT statement, SQLINTEGER attribute, SQLPOINTER value );
+int __check_stmt_from_dbc( DMHDBC connection, int state );
+int __check_stmt_from_desc( DMHDESC desc, int state );
+int __check_stmt_from_desc_ird( DMHDESC desc, int state );
 
 /* 
  * These are passed to the cursor lib as helper functions
@@ -704,6 +725,12 @@ void thread_release( int type, void *handle );
 
 void dbc_change_thread_support( DMHDBC connection, int level );
 
+#ifdef WITH_HANDLE_REDIRECT
+
+void *find_parent_handle( DRV_SQLHANDLE hand, int type );
+
+#endif
+
 /*
  * lookup functions
  */
@@ -1376,4 +1403,10 @@ void return_to_pool( DMHDBC connection );
                                     (con->functions[77].funcW)\
                                         (typ,han,rn,st,nat,msg,bl,tlp)
 
+#define DM_SQLCANCELHANDLE          78
+#define CHECK_SQLCANCELHANDLE(con)  (con->functions[78].func!=NULL)
+#define SQLCANCELHANDLE(con,typ,han)\
+                                    (con->functions[78].func)\
+                                        (typ,han)
+
 #endif
