@@ -23,9 +23,15 @@
  *
  **********************************************************************
  *
- * $Id: SQLConnect.c,v 1.5 2005/07/08 12:11:23 lurcher Exp $
+ * $Id: SQLConnect.c,v 1.7 2009/02/18 17:59:17 lurcher Exp $
  *
  * $Log: SQLConnect.c,v $
+ * Revision 1.7  2009/02/18 17:59:17  lurcher
+ * Shift to using config.h, the compile lines were making it hard to spot warnings
+ *
+ * Revision 1.6  2007/02/12 11:49:35  lurcher
+ * Add QT4 support to existing GUI parts
+ *
  * Revision 1.5  2005/07/08 12:11:23  lurcher
  *
  * Fix a cursor lib problem (it was broken if you did metadata calls)
@@ -75,6 +81,7 @@
  *
  **********************************************************************/
 
+#include <config.h>
 #include "cursorlibrary.h"
 
 static struct driver_func  cl_template_func[] =
@@ -359,7 +366,7 @@ SQLRETURN CLConnect( DMHDBC connection, struct driver_helper_funcs *dh )
      */
 
     cl_connection -> driver_dbc = connection -> driver_dbc;
-    connection -> driver_dbc = ( SQLHANDLE ) cl_connection;
+    connection -> driver_dbc = ( DRV_SQLHANDLE ) cl_connection;
 
     /*
      * check the number of alowed active statements
