@@ -23,9 +23,24 @@
  *
  **********************************************************************
  *
- * $Id: SQLExtendedFetch.c,v 1.9 2005/10/27 17:54:49 lurcher Exp $
+ * $Id: SQLExtendedFetch.c,v 1.14 2009/02/18 17:59:17 lurcher Exp $
  *
  * $Log: SQLExtendedFetch.c,v $
+ * Revision 1.14  2009/02/18 17:59:17  lurcher
+ * Shift to using config.h, the compile lines were making it hard to spot warnings
+ *
+ * Revision 1.13  2009/02/17 09:47:45  lurcher
+ * Clear up a number of bugs
+ *
+ * Revision 1.12  2008/01/22 17:51:54  lurcher
+ * Another SQLULEN mismatch
+ *
+ * Revision 1.11  2007/11/29 12:00:36  lurcher
+ * Add 64 bit type changes to SQLExtendedFetch etc
+ *
+ * Revision 1.10  2007/11/13 15:04:57  lurcher
+ * Fix 64 bit cursor lib issues
+ *
  * Revision 1.9  2005/10/27 17:54:49  lurcher
  * fix what I suspect is a typo in qt.m4
  *
@@ -85,6 +100,7 @@
  *
  **********************************************************************/
 
+#include <config.h>
 #include "cursorlibrary.h"
 
 #define ABS(x)  (((x)>=0)?(x):(-(x)))
@@ -195,7 +211,7 @@ SQLRETURN fetch_row( CLHSTMT cl_statement, int row_number, int offset )
                     if ( cbuf -> bound_ind )
                     {
                         ind_ptr = ( char * ) cbuf -> bound_ind;
-                        ind_ptr += offset * sizeof( SQLUINTEGER );
+                        ind_ptr += offset * sizeof( SQLULEN );
                     }
                 }
 
@@ -313,7 +329,7 @@ SQLRETURN fetch_row( CLHSTMT cl_statement, int row_number, int offset )
                         if ( cbuf -> bound_ind )
                         {
                             ind_ptr = ( char * ) cbuf -> bound_ind;
-                            ind_ptr += offset * sizeof( SQLUINTEGER );
+                            ind_ptr += offset * sizeof( SQLULEN );
                         }
                     }
 
@@ -453,7 +469,7 @@ static SQLRETURN fetch_rowset( CLHSTMT cl_statement,
         int row_offset,
         int *fetched_rows,
         SQLUSMALLINT *row_status_array,
-        SQLUINTEGER *rows_fetched_ptr )
+        SQLULEN *rows_fetched_ptr )
 {
     SQLRETURN ret;
     int row_count = 0;
@@ -505,9 +521,9 @@ static SQLRETURN fetch_rowset( CLHSTMT cl_statement,
 
 SQLRETURN do_fetch_scroll( CLHSTMT cl_statement,
         int fetch_orientation,
-        int fetch_offset,
+        SQLLEN fetch_offset,
         SQLUSMALLINT *row_status_ptr,
-        SQLUINTEGER *rows_fetched_ptr,
+        SQLULEN *rows_fetched_ptr,
 		int ext_fetch )
 {
     SQLRETURN ret;
@@ -603,7 +619,7 @@ SQLRETURN do_fetch_scroll( CLHSTMT cl_statement,
         }
 		else if ( ret == SQL_FETCH_PART_ROWSET ) 
 		{
-        	cl_statement -> rowset_position == CL_AFTER_END;
+        	cl_statement -> rowset_position = CL_AFTER_END;
 			ret = SQL_SUCCESS;
 		}
         break;
@@ -908,8 +924,8 @@ SQLRETURN do_fetch_scroll( CLHSTMT cl_statement,
 SQLRETURN CLExtendedFetch(
     SQLHSTMT           statement_handle,
     SQLUSMALLINT       f_fetch_type,
-    SQLINTEGER         irow,
-    SQLUINTEGER        *pcrow,
+    SQLLEN             irow,
+    SQLULEN            *pcrow,
     SQLUSMALLINT       *rgf_row_status )
 {
     CLHSTMT cl_statement = (CLHSTMT) statement_handle; 
